@@ -1,8 +1,24 @@
 /*
- * $Id: citadel.h 7525 2009-06-02 19:04:52Z ajc $
+ * $Id: citadel.h 8127 2009-12-16 21:27:59Z dothebart $
  *
  * Main Citadel header file
  * See COPYING for copyright information.
+ *
+ * Copyright (c) 1987-2009 by the citadel.org team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 /* system customizations are in sysconfig.h */
@@ -38,10 +54,10 @@ extern "C" {
  * usually more strict because you're not really supposed to dump/load and
  * upgrade at the same time.
  */
-#define REV_LEVEL	751		/* This version */
+#define REV_LEVEL	770		/* This version */
 #define REV_MIN		591		/* Oldest compatible database */
-#define EXPORT_REV_MIN	737		/* Oldest compatible export files */
-#define LIBCITADEL_MIN	744		/* Minimum required version of libcitadel */
+#define EXPORT_REV_MIN	760		/* Oldest compatible export files */
+#define LIBCITADEL_MIN	770		/* Minimum required version of libcitadel */
 
 #define SERVER_TYPE 0			/* zero for stock Citadel; other developers please
 					   obtain SERVER_TYPE codes for your implementations */
@@ -278,7 +294,8 @@ enum {
  */
 #define AUTHMODE_NATIVE		0	/* Native (self-contained or "black box") */
 #define AUTHMODE_HOST		1	/* Authenticate against the host OS user database */
-#define AUTHMODE_LDAP		2	/* External LDAP authentication (NOT IMPLEMENTED) */
+#define AUTHMODE_LDAP		2	/* Authenticate against an LDAP server with RFC 2307 schema */
+#define AUTHMODE_LDAP_AD	3	/* Authenticate against non-standard MS Active Directory LDAP */
 
 #ifdef __cplusplus
 }
