@@ -1,8 +1,23 @@
 /* 
- * $Id: citserver.c 7290 2009-04-06 12:14:42Z dothebart $
+ * $Id: citserver.c 8080 2009-11-24 20:19:06Z ajc $
  *
  * Main source module for the Citadel server
  *
+ * Copyright (c) 1987-2009 by the citadel.org team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "sysdep.h"
@@ -58,6 +73,7 @@
 #include "policy.h"
 #include "control.h"
 #include "euidindex.h"
+#include "context.h"
 #include "svn_revision.h"
 
 #ifndef HAVE_SNPRINTF
@@ -67,15 +83,13 @@
 #include "ctdl_module.h"
 
 
-struct CitContext *ContextList = NULL;
-struct CitContext* next_session = NULL;
 char *unique_session_numbers;
 int ScheduledShutdown = 0;
 time_t server_startup_time;
 int panic_fd;
 
-/**
- * \brief print the actual stack frame.
+/*
+ * print the actual stack frame.
  */
 void cit_backtrace(void)
 {
@@ -97,8 +111,8 @@ void cit_backtrace(void)
 #endif
 }
 
-/**
- * \brief print the actual stack frame.
+/*
+ * print the actual stack frame.
  */
 void cit_panic_backtrace(int SigNum)
 {
@@ -129,6 +143,7 @@ void master_startup(void) {
 	unsigned int seed;
 	FILE *urandom;
 	struct ctdlroom qrbuf;
+	int rv;
 	
 	CtdlLogPrintf(CTDL_DEBUG, "master_startup() started\n");
 	time(&server_startup_time);
@@ -143,27 +158,27 @@ void master_startup(void) {
 	check_ref_counts();
 
 	CtdlLogPrintf(CTDL_INFO, "Creating base rooms (if necessary)\n");
-	create_room(config.c_baseroom,	0, "", 0, 1, 0, VIEW_BBS);
-	create_room(AIDEROOM,		3, "", 0, 1, 0, VIEW_BBS);
-	create_room(SYSCONFIGROOM,	3, "", 0, 1, 0, VIEW_BBS);
-	create_room(config.c_twitroom,	0, "", 0, 1, 0, VIEW_BBS);
+	CtdlCreateRoom(config.c_baseroom,	0, "", 0, 1, 0, VIEW_BBS);
+	CtdlCreateRoom(AIDEROOM,		3, "", 0, 1, 0, VIEW_BBS);
+	CtdlCreateRoom(SYSCONFIGROOM,		3, "", 0, 1, 0, VIEW_BBS);
+	CtdlCreateRoom(config.c_twitroom,	0, "", 0, 1, 0, VIEW_BBS);
 
 	/* The "Local System Configuration" room doesn't need to be visible */
-        if (lgetroom(&qrbuf, SYSCONFIGROOM) == 0) {
+        if (CtdlGetRoomLock(&qrbuf, SYSCONFIGROOM) == 0) {
                 qrbuf.QRflags2 |= QR2_SYSTEM;
-                lputroom(&qrbuf);
+                CtdlPutRoomLock(&qrbuf);
         }
 
 	/* Aide needs to be public postable, else we're not RFC conformant. */
-        if (lgetroom(&qrbuf, AIDEROOM) == 0) {
+        if (CtdlGetRoomLock(&qrbuf, AIDEROOM) == 0) {
                 qrbuf.QRflags2 |= QR2_SMTP_PUBLIC;
-                lputroom(&qrbuf);
+                CtdlPutRoomLock(&qrbuf);
         }
 
 	CtdlLogPrintf(CTDL_INFO, "Seeding the pseudo-random number generator...\n");
 	urandom = fopen("/dev/urandom", "r");
 	if (urandom != NULL) {
-		fread(&seed, sizeof seed, 1, urandom);
+		rv = fread(&seed, sizeof seed, 1, urandom);
 		fclose(urandom);
 	}
 	else {
@@ -235,44 +250,10 @@ void master_cleanup(int exitcode) {
 
 
 
-/*
- * Terminate a session.
- */
-void RemoveContext (struct CitContext *con)
-{
-	if (con==NULL) {
-		CtdlLogPrintf(CTDL_ERR,
-			"WARNING: RemoveContext() called with NULL!\n");
-		return;
-	}
-	CtdlLogPrintf(CTDL_DEBUG, "RemoveContext() session %d\n", con->cs_pid);
-
-	/* Run any cleanup routines registered by loadable modules.
-	 * Note: We have to "become_session()" because the cleanup functions
-	 *       might make references to "CC" assuming it's the right one.
-	 */
-	become_session(con);
-	logout();
-	PerformSessionHooks(EVT_STOP);
-	become_session(NULL);
-
-	CtdlLogPrintf(CTDL_NOTICE, "[%3d] Session ended.\n", con->cs_pid);
-
-	/* If the client is still connected, blow 'em away. */
-	CtdlLogPrintf(CTDL_DEBUG, "Closing socket %d\n", con->client_socket);
-	close(con->client_socket);
-
-	CtdlLogPrintf(CTDL_DEBUG, "Done with RemoveContext()\n");
-}
-
-
-
-
-
 /*
  * cmd_info()  -  tell the client about this server
  */
-void cmd_info(void) {
+void cmd_info(char *cmdbuf) {
 	cprintf("%d Server info:\n", LISTING_FOLLOWS);
 	cprintf("%d\n", CC->cs_pid);
 	cprintf("%s\n", config.c_nodename);
@@ -337,7 +318,7 @@ char CtdlCheckExpress(void) {
 	}
 }
 
-void cmd_time(void)
+void cmd_time(char *argbuf)
 {
    time_t tv;
    struct tm tmp;
@@ -376,10 +357,7 @@ int is_public_client(void)
 
 #define LOCALHOSTSTR "127.0.0.1"
 
-	snprintf(public_clients_file, 
-			 sizeof public_clients_file,
-			 "%s/public_clients",
-			 ctdl_etc_dir);
+	snprintf(public_clients_file, sizeof public_clients_file, "%s/public_clients", ctdl_etc_dir);
 
 	/*
 	 * Check the time stamp on the public_clients file.  If it's been
@@ -515,7 +493,8 @@ void cmd_iden(char *argbuf)
 		(rev_level / 100),
 		(rev_level % 100),
 		desc,
-		CC->cs_host);
+		CC->cs_host
+	);
 	cprintf("%d Ok\n",CIT_OK);
 }
 
@@ -659,8 +638,8 @@ void cmd_emsg(char *mname)
  * user also knows the rooms.
  */
 void GenerateRoomDisplay(char *real_room,
-			struct CitContext *viewed,
-			struct CitContext *viewer) {
+			CitContext *viewed,
+			CitContext *viewer) {
 
 	int ra;
 
@@ -727,35 +706,19 @@ int CtdlAccessCheck(int required_level) {
 void cmd_term(char *cmdbuf)
 {
 	int session_num;
-	struct CitContext *ccptr;
-	int found_it = 0;
-	int allowed = 0;
+	int terminated = 0;
 
 	session_num = extract_int(cmdbuf, 0);
-	if (session_num == CC->cs_pid) {
+
+	terminated = CtdlTerminateOtherSession(session_num);
+
+	if (terminated < 0) {
 		cprintf("%d You can't kill your own session.\n", ERROR + ILLEGAL_VALUE);
 		return;
 	}
 
-	CtdlLogPrintf(CTDL_DEBUG, "Locating session to kill\n");
-	begin_critical_section(S_SESSION_TABLE);
-	for (ccptr = ContextList; ccptr != NULL; ccptr = ccptr->next) {
-		if (session_num == ccptr->cs_pid) {
-			found_it = 1;
-			if ((ccptr->user.usernum == CC->user.usernum)
-			   || (CC->user.axlevel >= 6)) {
-				allowed = 1;
-				ccptr->kill_me = 1;
-			}
-			else {
-				allowed = 0;
-			}
-		}
-	}
-	end_critical_section(S_SESSION_TABLE);
-
-	if (found_it) {
-		if (allowed) {
+	if (terminated & TERM_FOUND) {
+		if (terminated == TERM_KILLED) {
 			cprintf("%d Session terminated.\n", CIT_OK);
 		}
 		else {
@@ -769,16 +732,14 @@ void cmd_term(char *cmdbuf)
 }
 
 
-
-
-
 /* 
  * get the paginator prompt
  */
-void cmd_more(void) {
+void cmd_more(char *argbuf) {
 	cprintf("%d %s\n", CIT_OK, config.c_moreprompt);
 }
 
+
 /*
  * echo 
  */
@@ -788,9 +749,8 @@ void cmd_echo(char *etext)
 }
 
 
-
 /* 
- * identify as internal program
+ * Perform privilege escalation for an internal program
  */
 void cmd_ipgm(char *argbuf)
 {
@@ -803,8 +763,7 @@ void cmd_ipgm(char *argbuf)
 	 */
 	if (!CC->is_local_socket) {
 		sleep(5);
-		cprintf("%d Authentication failed.\n",
-			ERROR + PASSWORD_REQUIRED);
+		cprintf("%d Authentication failed.\n", ERROR + PASSWORD_REQUIRED);
 	}
 	else if (secret == config.c_ipgm_secret) {
 		CC->internal_pgm = 1;
@@ -814,8 +773,7 @@ void cmd_ipgm(char *argbuf)
 	}
 	else {
 		sleep(5);
-		cprintf("%d Authentication failed.\n",
-			ERROR + PASSWORD_REQUIRED);
+		cprintf("%d Authentication failed.\n", ERROR + PASSWORD_REQUIRED);
 		CtdlLogPrintf(CTDL_ERR, "Warning: ipgm authentication failed.\n");
 		CC->kill_me = 1;
 	}
@@ -851,13 +809,15 @@ void cmd_down(char *argbuf) {
 	{
 		cprintf(Reply, CIT_OK + SERVER_SHUTTING_DOWN); 
 	}
+	CC->kill_me = 1; /* Even the DOWN command has to follow correct proceedure when disconecting */
 	CtdlThreadStopAll();
 }
 
+
 /*
  * Halt the server without exiting the server process.
  */
-void cmd_halt(void) {
+void cmd_halt(char *argbuf) {
 
 	if (CtdlAccessCheck(ac_aide)) return;
 
@@ -866,6 +826,7 @@ void cmd_halt(void) {
 	shutdown_and_halt = 1;
 }
 
+
 /*
  * Schedule or cancel a server shutdown
  */
@@ -919,7 +880,7 @@ void cmd_asyn(char *argbuf)
  * RFC 1725 et al specify a PID to be placed in front of the nonce.
  * Quoth BTX: That would be stupid.
  */
-void generate_nonce(struct CitContext *con) {
+void generate_nonce(CitContext *con) {
 	struct timeval tv;
 
 	memset(con->cs_nonce, NONCE_SIZE, 0);
@@ -930,12 +891,10 @@ void generate_nonce(struct CitContext *con) {
 }
 
 
-
-
 /*
  * Back-end function for starting a session
  */
-void begin_session(struct CitContext *con)
+void begin_session(CitContext *con)
 {
 	socklen_t len;
 	struct sockaddr_in sin;
@@ -960,6 +919,7 @@ void begin_session(struct CitContext *con)
 	generate_nonce(con);
 	safestrncpy(con->cs_host, config.c_fqdn, sizeof con->cs_host);
 	safestrncpy(con->cs_addr, "", sizeof con->cs_addr);
+	con->cs_UDSclientUID = -1;
 	con->cs_host[sizeof con->cs_host - 1] = 0;
 	len = sizeof sin;
 	if (!CC->is_local_socket) {
@@ -972,6 +932,34 @@ void begin_session(struct CitContext *con)
 	}
 	else {
 		strcpy(con->cs_host, "");
+#ifdef HAVE_STRUCT_UCRED
+		{
+			/* as http://www.wsinnovations.com/softeng/articles/uds.html told us... */
+			struct ucred credentials;
+			socklen_t ucred_length = sizeof(struct ucred);
+			
+			/*fill in the user data structure */
+			if(getsockopt(con->client_socket, SOL_SOCKET, SO_PEERCRED, &credentials, &ucred_length)) {
+				CtdlLogPrintf(CTDL_NOTICE, "could obtain credentials from unix domain socket");
+				
+			}
+			else {		
+				/* the process ID of the process on the other side of the socket */
+				/* credentials.pid; */
+				
+				/* the effective UID of the process on the other side of the socket  */
+				con->cs_UDSclientUID = credentials.uid;
+				
+				/* the effective primary GID of the process on the other side of the socket */
+				/* credentials.gid; */
+				
+				/* To get supplemental groups, we will have to look them up in our account
+				   database, after a reverse lookup on the UID to get the account name.
+				   We can take this opportunity to check to see if this is a legit account.
+				*/
+			}
+		}
+#endif
 	}
 	con->cs_flags = 0;
 	con->upload_type = UPL_FILE;
@@ -1003,12 +991,37 @@ void citproto_begin_session() {
 		CC->kill_me = 1;
 	}
 	else {
-		cprintf("%d %s Citadel server ready.\n",
-			CIT_OK, config.c_nodename);
+		cprintf("%d %s Citadel server ready.\n", CIT_OK, config.c_nodename);
+		CC->can_receive_im = 1;
 	}
 }
 
 
+void cmd_noop(char *argbuf)
+{
+	cprintf("%d%cok\n", CIT_OK, CtdlCheckExpress() );
+}
+
+
+void cmd_qnop(char *argbuf)
+{
+	/* do nothing, this command returns no response */
+}
+
+
+void cmd_quit(char *argbuf)
+{
+	cprintf("%d Goodbye.\n", CIT_OK);
+	CC->kill_me = 1;
+}
+
+
+void cmd_lout(char *argbuf)
+{
+	if (CC->logged_in) 
+		CtdlUserLogout();
+	cprintf("%d logged out.\n", CIT_OK);
+}
 
 
 /*
@@ -1064,367 +1077,9 @@ void do_command_loop(void) {
 	   CC->cs_flags &= ~CS_POSTING;
 	}
 		   
-	if (!strncasecmp(cmdbuf, "NOOP", 4)) {
-		cprintf("%d%cok\n", CIT_OK, CtdlCheckExpress() );
-	}
-	
-	else if (!strncasecmp(cmdbuf, "XYZZY", 5)) {
-		cprintf("%d Nothing happens.\n", CIT_OK);
-	}
-	
-	else if (!strncasecmp(cmdbuf, "QNOP", 4)) {
-		/* do nothing, this command returns no response */
-	}
-
-	else if (!strncasecmp(cmdbuf,"QUIT",4)) {
-		cprintf("%d Goodbye.\n", CIT_OK);
-		CC->kill_me = 1;
-	}
-
-	else if (!strncasecmp(cmdbuf,"ASYN",4)) {
-		cmd_asyn(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"LOUT",4)) {
-		if (CC->logged_in) logout();
-		cprintf("%d logged out.\n", CIT_OK);
-	}
-
-	else if (!strncasecmp(cmdbuf,"USER",4)) {
-		cmd_user(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"PASS",4)) {
-		cmd_pass(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"NEWU",4)) {
-		cmd_newu(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"CREU",4)) {
-		cmd_creu(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"SETP",4)) {
-		cmd_setp(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"LRMS",4)) {
-		cmd_lrms(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"LKRA",4)) {
-		cmd_lkra(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"LKRN",4)) {
-		cmd_lkrn(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"LKRO",4)) {
-		cmd_lkro(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"LZRM",4)) {
-		cmd_lzrm(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"LPRM",4)) {
-		cmd_lprm(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"GETU",4)) {
-		cmd_getu();
-	}
-
-	else if (!strncasecmp(cmdbuf,"SETU",4)) {
-		cmd_setu(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"GOTO",4)) {
-		cmd_goto(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"MSGS",4)) {
-		cmd_msgs(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"WHOK",4)) {
-		cmd_whok();
-	}
-
-	else if (!strncasecmp(cmdbuf,"RDIR",4)) {
-		cmd_rdir();
-	}
-
-	else if (!strncasecmp(cmdbuf,"EUID",4)) {
-		cmd_euid(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"MSG0",4)) {
-		cmd_msg0(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"MSG2",4)) {
-		cmd_msg2(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"MSG3",4)) {
-		cmd_msg3(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"MSG4",4)) {
-		cmd_msg4(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"MSGP",4)) {
-		cmd_msgp(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"OPNA",4)) {
-		cmd_opna(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"DLAT",4)) {
-		cmd_dlat(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"INFO",4)) {
-		cmd_info();
-	}
-
-	else if (!strncasecmp(cmdbuf,"SLRP",4)) {
-		cmd_slrp(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"INVT",4)) {
-		cmd_invt_kick(&cmdbuf[5],1);
-	}
-
-	else if (!strncasecmp(cmdbuf,"KICK",4)) {
-		cmd_invt_kick(&cmdbuf[5],0);
-	}
-
-	else if (!strncasecmp(cmdbuf,"GETR",4)) {
-		cmd_getr();
-	}
-
-	else if (!strncasecmp(cmdbuf,"SETR",4)) {
-		cmd_setr(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"GETA",4)) {
-		cmd_geta();
-	}
-
-	else if (!strncasecmp(cmdbuf,"SETA",4)) {
-		cmd_seta(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"ENT0",4)) {
-		cmd_ent0(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"RINF",4)) {
-		cmd_rinf();
-	}
-
-	else if (!strncasecmp(cmdbuf,"DELE",4)) {
-		cmd_dele(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"KILL",4)) {
-		cmd_kill(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"CRE8",4)) {
-		cmd_cre8(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"MOVE",4)) {
-		cmd_move(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"FORG",4)) {
-		cmd_forg();
-	}
-
-	else if (!strncasecmp(cmdbuf,"MESG",4)) {
-		cmd_mesg(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"EMSG",4)) {
-		cmd_emsg(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"GNUR",4)) {
-		cmd_gnur();
-	}
-
-	else if (!strncasecmp(cmdbuf,"VALI",4)) {
-		cmd_vali(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"EINF",4)) {
-		cmd_einf(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"LIST",4)) {
-		cmd_list(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"CHEK",4)) {
-		cmd_chek();
-	}
-
-	else if (!strncasecmp(cmdbuf,"DELF",4)) {
-		cmd_delf(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"MOVF",4)) {
-		cmd_movf(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"OPEN",4)) {
-		cmd_open(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"CLOS",4)) {
-		cmd_clos();
-	}
-
-	else if (!strncasecmp(cmdbuf,"UOPN",4)) {
-		cmd_uopn(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"UCLS",4)) {
-		cmd_ucls(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"READ",4)) {
-		cmd_read(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"WRIT",4)) {
-		cmd_writ(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"QUSR",4)) {
-		cmd_qusr(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"ECHO",4)) {
-		cmd_echo(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"OIMG",4)) {
-		cmd_oimg(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"MORE",4)) {
-		cmd_more();
-	}
-
-	else if (!strncasecmp(cmdbuf,"NDOP",4)) {
-		cmd_ndop(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"NUOP",4)) {
-		cmd_nuop(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"LFLR",4)) {
-		cmd_lflr();
-	}
-
-	else if (!strncasecmp(cmdbuf,"CFLR",4)) {
-		cmd_cflr(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"KFLR",4)) {
-		cmd_kflr(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"EFLR",4)) {
-		cmd_eflr(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"IDEN",4)) {
-		cmd_iden(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"IPGM",4)) {
-		cmd_ipgm(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"TERM",4)) {
-		cmd_term(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"DOWN",4)) {
-		cmd_down(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf,"HALT",4)) {
-		cmd_halt();
-	}
-
-	else if (!strncasecmp(cmdbuf,"SCDN",4)) {
-		cmd_scdn(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf, "UIMG", 4)) {
-		cmd_uimg(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf, "TIME", 4)) {
-		cmd_time();
-	}
-
-	else if (!strncasecmp(cmdbuf, "AGUP", 4)) {
-		cmd_agup(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf, "ASUP", 4)) {
-		cmd_asup(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf, "GPEX", 4)) {
-		cmd_gpex(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf, "SPEX", 4)) {
-		cmd_spex(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf, "CONF", 4)) {
-		cmd_conf(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf, "SEEN", 4)) {
-		cmd_seen(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf, "GTSN", 4)) {
-		cmd_gtsn(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf, "VIEW", 4)) {
-		cmd_view(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf, "ISME", 4)) {
-		cmd_isme(&cmdbuf[5]);
-	}
-
-	else if (!strncasecmp(cmdbuf, "RENU", 4)) {
-		cmd_renu(&cmdbuf[5]);
-	}
-
-	else if (!DLoader_Exec_Cmd(cmdbuf)) {
+	if (!DLoader_Exec_Cmd(cmdbuf)) {
 		cprintf("%d Unrecognized or unsupported command.\n", ERROR + CMD_NOT_SUPPORTED);
-	}
+	}	
 
 	unbuffer_output();
 
@@ -1440,3 +1095,33 @@ void do_command_loop(void) {
 void do_async_loop(void) {
 	PerformSessionHooks(EVT_ASYNC);
 }
+
+
+/*****************************************************************************/
+/*                      MODULE INITIALIZATION STUFF                          */
+/*****************************************************************************/
+
+CTDL_MODULE_INIT(citserver)
+{
+	if (!threading) {
+		CtdlRegisterProtoHook(cmd_noop, "NOOP", "no operation");
+		CtdlRegisterProtoHook(cmd_qnop, "QNOP", "no operation with no response");
+		CtdlRegisterProtoHook(cmd_quit, "QUIT", "log out and disconnect from server");
+		CtdlRegisterProtoHook(cmd_lout, "LOUT", "log out but do not disconnect from server");
+		CtdlRegisterProtoHook(cmd_asyn, "ASYN", "enable asynchronous server responses");
+		CtdlRegisterProtoHook(cmd_info, "INFO", "fetch server capabilities and configuration");
+		CtdlRegisterProtoHook(cmd_mesg, "MESG", "fetch system banners");
+		CtdlRegisterProtoHook(cmd_emsg, "EMSG", "submit system banners");
+		CtdlRegisterProtoHook(cmd_echo, "ECHO", "echo text back to the client");
+		CtdlRegisterProtoHook(cmd_more, "MORE", "fetch the paginator prompt");
+		CtdlRegisterProtoHook(cmd_iden, "IDEN", "identify the client software and location");
+		CtdlRegisterProtoHook(cmd_ipgm, "IPGM", "perform privilege escalation for internal programs");
+		CtdlRegisterProtoHook(cmd_term, "TERM", "terminate another running session");
+		CtdlRegisterProtoHook(cmd_down, "DOWN", "perform a server shutdown");
+		CtdlRegisterProtoHook(cmd_halt, "HALT", "halt the server without exiting the server process");
+		CtdlRegisterProtoHook(cmd_scdn, "SCDN", "schedule or cancel a server shutdown");
+		CtdlRegisterProtoHook(cmd_time, "TIME", "fetch the date and time from the server");
+	}
+        /* return our Subversion id for the Log */
+	return "$Id: citserver.c 8080 2009-11-24 20:19:06Z ajc $";
+}
