@@ -1,9 +1,23 @@
 /*
- * $Id: config.c 7243 2009-03-20 16:36:12Z ajc $
+ * $Id: config.c 7954 2009-10-24 23:23:27Z davew $
  *
- * This function reads the citadel.config file.  It should be called at
- * the beginning of EVERY Citadel program.
+ * Read and write the citadel.config file
  *
+ * Copyright (c) 1987-2009 by the citadel.org team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "sysdep.h"
@@ -19,6 +33,10 @@
 #include "server.h"
 #include "config.h"
 
+
+#include "ctdl_module.h"
+
+
 struct config config;
 
 /*
@@ -79,6 +97,14 @@ void get_config(void) {
 		exit(CTDLEXIT_LIBCITADEL);
 	}
 
+	/* Only allow LDAP auth mode if we actually have LDAP support */
+#ifndef HAVE_LDAP
+	if ((config.c_auth_mode == AUTHMODE_LDAP) || (config.c_auth_mode == AUTHMODE_LDAP_AD)) {
+		fprintf(stderr, "Your system is configured for LDAP authentication,\n"
+				"but you are running a server built without OpenLDAP support.\n");
+		exit(CTDL_EXIT_UNSUP_AUTH);
+	}
+#endif
 
 	/* Check to see whether 'setup' must first be run to update data file formats */
 	if (config.c_setup_level < REV_MIN) {
