@@ -1,8 +1,23 @@
 /*
- * $Id: control.c 7380 2009-04-19 18:24:56Z dothebart $
+ * $Id: control.c 7979 2009-10-29 17:40:39Z davew $
  *
  * This module handles states which are global to the entire server.
  *
+ * Copyright (c) 1987-2009 by the citadel.org team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "sysdep.h"
@@ -48,6 +63,8 @@
 #include "snprintf.h"
 #endif
 
+#include "ctdl_module.h"
+
 struct CitControl CitControl;
 extern struct config config;
 FILE *control_fp = NULL;
@@ -93,7 +110,7 @@ void control_find_highest(struct ctdlroom *qrbuf, void *data)
 		room_fixed = 1;
 	}
 		
-	getroom (&room, qrbuf->QRname);
+	CtdlGetRoom (&room, qrbuf->QRname);
 	
 	/* Load the message list */
 	cdbfr = cdb_fetch(CDB_MSGLISTS, &room.QRnumber, sizeof(long));
@@ -148,6 +165,7 @@ void control_find_user (struct ctdluser *EachUser, void *out_data)
 void get_control(void)
 {
 	static int already_have_control = 0;
+	int rv = 0;
 
 	/*
 	 * If we already have the control record in memory, there's no point
@@ -164,35 +182,30 @@ void get_control(void)
 		control_fp = fopen(file_citadel_control, "rb+");
 		if (control_fp != NULL) {
 			lock_control();
-			fchown(fileno(control_fp), config.c_ctdluid, -1);
-			fchmod(fileno(control_fp), 
-			       S_IRUSR|S_IWUSR);
+			rv = fchown(fileno(control_fp), config.c_ctdluid, -1);
+			rv = fchmod(fileno(control_fp), S_IRUSR|S_IWUSR);
 		}
 	}
 	if (control_fp == NULL) {
 		control_fp = fopen(file_citadel_control, "wb+");
 		if (control_fp != NULL) {
 			lock_control();
-			fchown(fileno(control_fp), config.c_ctdluid, -1);
-			fchmod(fileno(control_fp), 
-			       S_IRUSR|S_IWUSR);
+			rv = fchown(fileno(control_fp), config.c_ctdluid, -1);
+			rv = fchmod(fileno(control_fp), S_IRUSR|S_IWUSR);
 			memset(&CitControl, 0, sizeof(struct CitControl));
-			fwrite(&CitControl, sizeof(struct CitControl),
-			       1, control_fp);
+			rv = fwrite(&CitControl, sizeof(struct CitControl), 1, control_fp);
 			rewind(control_fp);
 		}
 	}
 	if (control_fp == NULL) {
-		CtdlLogPrintf(CTDL_ALERT, "ERROR opening %s: %s\n",
-				file_citadel_control,
-				strerror(errno));
+		CtdlLogPrintf(CTDL_ALERT, "ERROR opening %s: %s\n", file_citadel_control, strerror(errno));
 		return;
 	}
 
 	rewind(control_fp);
-	fread(&CitControl, sizeof(struct CitControl), 1, control_fp);
+	rv = fread(&CitControl, sizeof(struct CitControl), 1, control_fp);
 	already_have_control = 1;
-	chown(file_citadel_control, config.c_ctdluid, (-1));
+	rv = chown(file_citadel_control, config.c_ctdluid, (-1));
 	
 }
 
@@ -201,11 +214,11 @@ void get_control(void)
  */
 void put_control(void)
 {
+	int rv = 0;
 
 	if (control_fp != NULL) {
 		rewind(control_fp);
-		fwrite(&CitControl, sizeof(struct CitControl), 1,
-		       control_fp);
+		rv = fwrite(&CitControl, sizeof(struct CitControl), 1, control_fp);
 		fflush(control_fp);
 	}
 }
@@ -219,7 +232,7 @@ void check_control(void)
 	CtdlLogPrintf(CTDL_INFO, "Checking/re-building control record\n");
 	get_control();
 	// Find highest room number and message number.
-	ForEachRoom(control_find_highest, NULL);
+	CtdlForEachRoom(control_find_highest, NULL);
 	ForEachUser(control_find_user, NULL);
 	put_control();
 }
@@ -653,10 +666,10 @@ void cmd_conf(char *argbuf)
 		snprintf(buf, sizeof buf,
 			 "The global system configuration has been edited by %s.\n",
 			 CC->curr_user);
-		aide_message(buf,"Citadel Configuration Manager Message");
+		CtdlAideMessage(buf,"Citadel Configuration Manager Message");
 
 		if (!IsEmptyStr(config.c_logpages))
-			create_room(config.c_logpages, 3, "", 0, 1, 1, VIEW_BBS);
+			CtdlCreateRoom(config.c_logpages, 3, "", 0, 1, 1, VIEW_BBS);
 
 		/* If full text indexing has been disabled, invalidate the
 		 * index so it doesn't try to use it later.
@@ -697,3 +710,18 @@ void cmd_conf(char *argbuf)
 			ERROR + ILLEGAL_VALUE);
 	}
 }
+
+
+/*****************************************************************************/
+/*                      MODULE INITIALIZATION STUFF                          */
+/*****************************************************************************/
+
+
+CTDL_MODULE_INIT(control)
+{
+	if (!threading) {
+		CtdlRegisterProtoHook(cmd_conf, "CONF", "Autoconverted. TODO: document me.");
+	}
+	/* return our Subversion id for the Log */
+	return "$Id: control.c 7979 2009-10-29 17:40:39Z davew $";
+}
