@@ -1,8 +1,24 @@
 /*
- * $Id: database.c 7205 2009-03-11 15:00:20Z ajc $
+ * $Id: database.c 7940 2009-10-23 16:22:11Z davew $
  *
  * This is a data store backend for the Citadel server which uses Berkeley DB.
  *
+ * Copyright (c) 1987-2009 by the citadel.org team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
  */
 
 /*****************************************************************************
@@ -90,8 +106,7 @@ static void txabort(DB_TXN * tid)
 	ret = tid->abort(tid);
 
 	if (ret) {
-		CtdlLogPrintf(CTDL_EMERG, "cdb_*: txn_abort: %s\n",
-			db_strerror(ret));
+		CtdlLogPrintf(CTDL_EMERG, "bdb(): txn_abort: %s\n", db_strerror(ret));
 		abort();
 	}
 }
@@ -104,8 +119,7 @@ static void txcommit(DB_TXN * tid)
 	ret = tid->commit(tid, 0);
 
 	if (ret) {
-		CtdlLogPrintf(CTDL_EMERG, "cdb_*: txn_commit: %s\n",
-			db_strerror(ret));
+		CtdlLogPrintf(CTDL_EMERG, "bdb(): txn_commit: %s\n", db_strerror(ret));
 		abort();
 	}
 }
@@ -118,15 +132,14 @@ static void txbegin(DB_TXN ** tid)
 	ret = dbenv->txn_begin(dbenv, NULL, tid, 0);
 
 	if (ret) {
-		CtdlLogPrintf(CTDL_EMERG, "cdb_*: txn_begin: %s\n",
-			db_strerror(ret));
+		CtdlLogPrintf(CTDL_EMERG, "bdb(): txn_begin: %s\n", db_strerror(ret));
 		abort();
 	}
 }
 
 static void dbpanic(DB_ENV * env, int errval)
 {
-	CtdlLogPrintf(CTDL_EMERG, "cdb_*: Berkeley DB panic: %d\n", errval);
+	CtdlLogPrintf(CTDL_EMERG, "bdb(): PANIC: %s\n", db_strerror(errval));
 }
 
 static void cclose(DBC * cursor)
@@ -134,8 +147,7 @@ static void cclose(DBC * cursor)
 	int ret;
 
 	if ((ret = cursor->c_close(cursor))) {
-		CtdlLogPrintf(CTDL_EMERG, "cdb_*: c_close: %s\n",
-			db_strerror(ret));
+		CtdlLogPrintf(CTDL_EMERG, "bdb(): c_close: %s\n", db_strerror(ret));
 		abort();
 	}
 }
@@ -147,8 +159,7 @@ static void bailIfCursor(DBC ** cursors, const char *msg)
 	for (i = 0; i < MAXCDB; i++)
 		if (cursors[i] != NULL) {
 			CtdlLogPrintf(CTDL_EMERG,
-				"cdb_*: cursor still in progress on cdb %02x: %s\n",
-				i, msg);
+				"bdb(): cursor still in progress on cdb %02x: %s\n", i, msg);
 			abort();
 		}
 }
@@ -161,8 +172,7 @@ void check_handles(void *arg)
 		bailIfCursor(tsd->cursors, "in check_handles");
 
 		if (tsd->tid != NULL) {
-			CtdlLogPrintf(CTDL_EMERG,
-				"cdb_*: transaction still in progress!");
+			CtdlLogPrintf(CTDL_EMERG, "bdb(): transaction still in progress!");
 			abort();
 		}
 	}
@@ -206,7 +216,7 @@ static void cdb_cull_logs(void)
 					 " This log file is no longer in use "
 					 "and may be safely deleted.\n",
 					 *file, strerror(errno));
-				aide_message(errmsg, "Database Warning Message");
+				CtdlAideMessage(errmsg, "Database Warning Message");
 			}
 		}
 		free(list);
@@ -231,13 +241,10 @@ void cdb_checkpoint(void)
 	int ret;
 
 	CtdlLogPrintf(CTDL_DEBUG, "-- db checkpoint --\n");
-	ret = dbenv->txn_checkpoint(dbenv,
-				    MAX_CHECKPOINT_KBYTES,
-				    MAX_CHECKPOINT_MINUTES, 0);
+	ret = dbenv->txn_checkpoint(dbenv, MAX_CHECKPOINT_KBYTES, MAX_CHECKPOINT_MINUTES, 0);
 
 	if (ret != 0) {
-		CtdlLogPrintf(CTDL_EMERG, "cdb_checkpoint: txn_checkpoint: %s\n",
-			db_strerror(ret));
+		CtdlLogPrintf(CTDL_EMERG, "cdb_checkpoint: txn_checkpoint: %s\n", db_strerror(ret));
 		abort();
 	}
 
@@ -264,7 +271,7 @@ void open_databases(void)
 	int dbversion_major, dbversion_minor, dbversion_patch;
 	int current_dbversion = 0;
 
-	CtdlLogPrintf(CTDL_DEBUG, "cdb_*: open_databases() starting\n");
+	CtdlLogPrintf(CTDL_DEBUG, "bdb(): open_databases() starting\n");
 	CtdlLogPrintf(CTDL_DEBUG, "Compiled db: %s\n", DB_VERSION_STRING);
 	CtdlLogPrintf(CTDL_INFO, "  Linked db: %s\n",
 		db_version(&dbversion_major, &dbversion_minor, &dbversion_patch));
@@ -298,11 +305,11 @@ void open_databases(void)
 	chmod(ctdl_data_dir, 0700);
 	chown(ctdl_data_dir, CTDLUID, (-1));
 
-	CtdlLogPrintf(CTDL_DEBUG, "cdb_*: Setting up DB environment\n");
+	CtdlLogPrintf(CTDL_DEBUG, "bdb(): Setting up DB environment\n");
 	db_env_set_func_yield(sched_yield);
 	ret = db_env_create(&dbenv, 0);
 	if (ret) {
-		CtdlLogPrintf(CTDL_EMERG, "cdb_*: db_env_create: %s\n", db_strerror(ret));
+		CtdlLogPrintf(CTDL_EMERG, "bdb(): db_env_create: %s\n", db_strerror(ret));
 		CtdlLogPrintf(CTDL_EMERG, "exit code %d\n", ret);
 		exit(CTDLEXIT_DB);
 	}
@@ -322,14 +329,14 @@ void open_databases(void)
 	 */
 	ret = dbenv->set_cachesize(dbenv, 0, 64 * 1024, 0);
 	if (ret) {
-		CtdlLogPrintf(CTDL_EMERG, "cdb_*: set_cachesize: %s\n", db_strerror(ret));
+		CtdlLogPrintf(CTDL_EMERG, "bdb(): set_cachesize: %s\n", db_strerror(ret));
 		dbenv->close(dbenv, 0);
 		CtdlLogPrintf(CTDL_EMERG, "exit code %d\n", ret);
 		exit(CTDLEXIT_DB);
 	}
 
 	if ((ret = dbenv->set_lk_detect(dbenv, DB_LOCK_DEFAULT))) {
-		CtdlLogPrintf(CTDL_EMERG, "cdb_*: set_lk_detect: %s\n", db_strerror(ret));
+		CtdlLogPrintf(CTDL_EMERG, "bdb(): set_lk_detect: %s\n", db_strerror(ret));
 		dbenv->close(dbenv, 0);
 		CtdlLogPrintf(CTDL_EMERG, "exit code %d\n", ret);
 		exit(CTDLEXIT_DB);
@@ -382,7 +389,8 @@ void open_databases(void)
 				   NULL,
 				   DB_BTREE,
 				   DB_CREATE | DB_AUTO_COMMIT | DB_THREAD,
-				   0600);
+				   0600
+		);
 		if (ret) {
 			CtdlLogPrintf(CTDL_EMERG, "db_open[%02x]: %s\n", i, db_strerror(ret));
 			if (ret == ENOMEM) {
@@ -422,7 +430,6 @@ void cdb_chmod_data(void) {
 	}
 
 	CtdlLogPrintf(CTDL_DEBUG, "open_databases() finished\n");
-
 	CtdlRegisterProtoHook(cmd_cull, "CULL", "Cull database logs");
 }
 
@@ -453,8 +460,7 @@ void close_databases(void)
 		CtdlLogPrintf(CTDL_INFO, "Closing database %02x\n", a);
 		ret = dbp[a]->close(dbp[a], 0);
 		if (ret) {
-			CtdlLogPrintf(CTDL_EMERG,
-				"db_close: %s\n", db_strerror(ret));
+			CtdlLogPrintf(CTDL_EMERG, "db_close: %s\n", db_strerror(ret));
 		}
 
 	}
@@ -462,8 +468,7 @@ void close_databases(void)
 	/* Close the handle. */
 	ret = dbenv->close(dbenv, 0);
 	if (ret) {
-		CtdlLogPrintf(CTDL_EMERG,
-			"DBENV->close: %s\n", db_strerror(ret));
+		CtdlLogPrintf(CTDL_EMERG, "DBENV->close: %s\n", db_strerror(ret));
 	}
 }
 
@@ -554,19 +559,15 @@ int cdb_store(int cdb, void *ckey, int ckeylen, void *cdata, int cdatalen)
 		    + sizeof(struct CtdlCompressHeader);
 		destLen = (uLongf) buffer_len;
 		compressed_data = malloc(buffer_len);
-		if (compress2((Bytef *) (compressed_data +
-					 sizeof(struct
-						CtdlCompressHeader)),
-			      &destLen, (Bytef *) cdata, (uLongf) cdatalen,
-			      1) != Z_OK) {
+		if (compress2((Bytef *) (compressed_data + sizeof(struct CtdlCompressHeader)),
+			&destLen, (Bytef *) cdata, (uLongf) cdatalen, 1) != Z_OK)
+		{
 			CtdlLogPrintf(CTDL_EMERG, "compress2() error\n");
 			abort();
 		}
 		zheader.compressed_len = (size_t) destLen;
-		memcpy(compressed_data, &zheader,
-		       sizeof(struct CtdlCompressHeader));
-		ddata.size = (size_t) (sizeof(struct CtdlCompressHeader) +
-				       zheader.compressed_len);
+		memcpy(compressed_data, &zheader, sizeof(struct CtdlCompressHeader));
+		ddata.size = (size_t) (sizeof(struct CtdlCompressHeader) + zheader.compressed_len);
 		ddata.data = compressed_data;
 	}
 #endif
@@ -578,8 +579,7 @@ int cdb_store(int cdb, void *ckey, int ckeylen, void *cdata, int cdatalen)
 				    &ddata,	/* data */
 				    0);	/* flags */
 		if (ret) {
-			CtdlLogPrintf(CTDL_EMERG, "cdb_store(%d): %s\n", cdb,
-				db_strerror(ret));
+			CtdlLogPrintf(CTDL_EMERG, "cdb_store(%d): %s\n", cdb, db_strerror(ret));
 			abort();
 		}
 #ifdef HAVE_ZLIB
@@ -589,8 +589,7 @@ int cdb_store(int cdb, void *ckey, int ckeylen, void *cdata, int cdatalen)
 		return ret;
 
 	} else {
-		bailIfCursor(MYCURSORS,
-			     "attempt to write during r/o cursor");
+		bailIfCursor(MYCURSORS, "attempt to write during r/o cursor");
 
 	      retry:
 		txbegin(&tid);
@@ -611,8 +610,9 @@ int cdb_store(int cdb, void *ckey, int ckeylen, void *cdata, int cdatalen)
 		} else {
 			txcommit(tid);
 #ifdef HAVE_ZLIB
-			if (compressing)
+			if (compressing) {
 				free(compressed_data);
+			}
 #endif
 			return ret;
 		}
@@ -637,14 +637,13 @@ int cdb_delete(int cdb, void *key, int keylen)
 	if (MYTID != NULL) {
 		ret = dbp[cdb]->del(dbp[cdb], MYTID, &dkey, 0);
 		if (ret) {
-			CtdlLogPrintf(CTDL_EMERG, "cdb_delete(%d): %s\n", cdb,
-				db_strerror(ret));
-			if (ret != DB_NOTFOUND)
+			CtdlLogPrintf(CTDL_EMERG, "cdb_delete(%d): %s\n", cdb, db_strerror(ret));
+			if (ret != DB_NOTFOUND) {
 				abort();
+			}
 		}
 	} else {
-		bailIfCursor(MYCURSORS,
-			     "attempt to delete during r/o cursor");
+		bailIfCursor(MYCURSORS, "attempt to delete during r/o cursor");
 
 	      retry:
 		txbegin(&tid);
@@ -724,8 +723,7 @@ struct cdbdata *cdb_fetch(int cdb, void *key, int keylen)
 	}
 
 	if ((ret != 0) && (ret != DB_NOTFOUND)) {
-		CtdlLogPrintf(CTDL_EMERG, "cdb_fetch(%d): %s\n", cdb,
-			db_strerror(ret));
+		CtdlLogPrintf(CTDL_EMERG, "cdb_fetch(%d): %s\n", cdb, db_strerror(ret));
 		abort();
 	}
 
@@ -734,8 +732,7 @@ struct cdbdata *cdb_fetch(int cdb, void *key, int keylen)
 	tempcdb = (struct cdbdata *) malloc(sizeof(struct cdbdata));
 
 	if (tempcdb == NULL) {
-		CtdlLogPrintf(CTDL_EMERG,
-			"cdb_fetch: Cannot allocate memory for tempcdb\n");
+		CtdlLogPrintf(CTDL_EMERG, "cdb_fetch: Cannot allocate memory for tempcdb\n");
 		abort();
 	}
 
@@ -764,8 +761,9 @@ void cdb_free(struct cdbdata *cdb)
 
 void cdb_close_cursor(int cdb)
 {
-	if (MYCURSORS[cdb] != NULL)
+	if (MYCURSORS[cdb] != NULL) {
 		cclose(MYCURSORS[cdb]);
+	}
 
 	MYCURSORS[cdb] = NULL;
 }
@@ -781,8 +779,7 @@ void cdb_rewind(int cdb)
 
 	if (MYCURSORS[cdb] != NULL) {
 		CtdlLogPrintf(CTDL_EMERG,
-			"cdb_rewind: must close cursor on database %d before reopening.\n",
-			cdb);
+			"cdb_rewind: must close cursor on database %d before reopening.\n", cdb);
 		abort();
 		/* cclose(MYCURSORS[cdb]); */
 	}
@@ -792,8 +789,7 @@ void cdb_rewind(int cdb)
 	 */
 	ret = dbp[cdb]->cursor(dbp[cdb], MYTID, &MYCURSORS[cdb], 0);
 	if (ret) {
-		CtdlLogPrintf(CTDL_EMERG, "cdb_rewind: db_cursor: %s\n",
-			db_strerror(ret));
+		CtdlLogPrintf(CTDL_EMERG, "cdb_rewind: db_cursor: %s\n", db_strerror(ret));
 		abort();
 	}
 }
@@ -818,8 +814,7 @@ struct cdbdata *cdb_next_item(int cdb)
 
 	if (ret) {
 		if (ret != DB_NOTFOUND) {
-			CtdlLogPrintf(CTDL_EMERG, "cdb_next_item(%d): %s\n",
-				cdb, db_strerror(ret));
+			CtdlLogPrintf(CTDL_EMERG, "cdb_next_item(%d): %s\n", cdb, db_strerror(ret));
 			abort();
 		}
 		cclose(MYCURSORS[cdb]);
@@ -844,12 +839,10 @@ struct cdbdata *cdb_next_item(int cdb)
 void cdb_begin_transaction(void)
 {
 
-	bailIfCursor(MYCURSORS,
-		     "can't begin transaction during r/o cursor");
+	bailIfCursor(MYCURSORS, "can't begin transaction during r/o cursor");
 
 	if (MYTID != NULL) {
-		CtdlLogPrintf(CTDL_EMERG,
-			"cdb_begin_transaction: ERROR: nested transaction\n");
+		CtdlLogPrintf(CTDL_EMERG, "cdb_begin_transaction: ERROR: nested transaction\n");
 		abort();
 	}
 
@@ -873,8 +866,9 @@ void cdb_end_transaction(void)
 		CtdlLogPrintf(CTDL_EMERG,
 			"cdb_end_transaction: ERROR: txcommit(NULL) !!\n");
 		abort();
-	} else
+	} else {
 		txcommit(MYTID);
+	}
 
 	MYTID = NULL;
 }
@@ -893,8 +887,7 @@ void cdb_trunc(int cdb)
 			"cdb_trunc must not be called in a transaction.\n");
 		abort();
 	} else {
-		bailIfCursor(MYCURSORS,
-			     "attempt to write during r/o cursor");
+		bailIfCursor(MYCURSORS, "attempt to write during r/o cursor");
 
 	      retry:
 		/* txbegin(&tid); */
