@@ -1,7 +1,9 @@
-/* $Id: serv_chat.h 5845 2007-12-03 04:56:31Z ajc $ */
+/* $Id: serv_chat.h 8004 2009-11-05 15:11:18Z davew $ */
+#include "ctdl_module.h"
+
 void ChatUnloadingTest(void);
 void allwrite (char *cmdbuf, int flag, char *username);
-t_context *find_context (char **unstr);
+CitContext *find_context (char **unstr);
 void do_chat_listing (int allflag);
 void cmd_chat (char *argbuf);
 void cmd_pexp (char *argbuf); /* arg unused */
@@ -9,8 +11,3 @@ void cmd_sexp (char *argbuf);
 void delete_instant_messages(void);
 void cmd_gexp(char *);
 int send_instant_message(char *, char *, char *, char *);
-
-struct savelist {
-	struct savelist *next;
-	char roomname[ROOMNAMELEN];
-};
