@@ -1,7 +1,23 @@
 /*
- * $Id: serv_checkpoint.c 5756 2007-11-16 17:15:22Z ajc $
+ * $Id: serv_checkpoint.c 7990 2009-11-01 18:09:14Z davew $
  *
  * checkpointing module for the database
+ *
+ * Copyright (c) 1987-2009 by the citadel.org team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
  
 #include "sysdep.h"
@@ -40,6 +56,7 @@
 #include "threads.h"
 
 #include "ctdl_module.h"
+#include "context.h"
  
 /*
  * Main loop for the checkpoint thread.
@@ -68,5 +85,5 @@ CTDL_MODULE_INIT(checkpoint) {
 		CtdlThreadCreate ("checkpoint", CTDLTHREAD_BIGSTACK, checkpoint_thread, NULL);
 	}
 	/* return our Subversion id for the Log */
-	return "$Id: serv_autocompletion.c 5756 2007-11-16 17:15:22Z ajc $";
+	return "$Id: serv_checkpoint.c 7990 2009-11-01 18:09:14Z davew $";
 }
