@@ -1,4 +1,21 @@
-/* $Id: serv_crypto.c 6281 2008-05-15 17:00:35Z ajc $ */
+/* $Id: serv_crypto.c 7914 2009-10-15 16:29:43Z ajc $
+ *
+ * Copyright (c) 1987-2009 by the citadel.org team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
 
 #include <string.h>
 #include <unistd.h>
@@ -56,8 +73,6 @@ static unsigned long id_callback(void)
 void destruct_ssl(void)
 {
 	int a;
-	CtdlUnregisterProtoHook(cmd_stls, "STLS");
-	CtdlUnregisterProtoHook(cmd_gtls, "GTLS");
 	for (a = 0; a < CRYPTO_num_locks(); a++) 
 		free(SSLCritters[a]);
 	free (SSLCritters);
