@@ -1,10 +1,24 @@
 /*
- * $Id: serv_dspam.c 5876 2007-12-10 23:22:03Z dothebart $
+ * $Id: serv_dspam.c 7921 2009-10-17 12:21:17Z davew $
  *
  * This module glues libDSpam to the Citadel server in order to implement
  * DSPAM Spamchecking 
  *
- * This code is released under the terms of the GNU General Public License. 
+ * Copyright (c) 2009 by the citadel.org team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "sysdep.h"
@@ -37,7 +51,6 @@
 #include "citserver.h"
 #include "support.h"
 #include "config.h"
-#include "room_ops.h"
 #include "policy.h"
 #include "database.h"
 #include "msgbase.h"
@@ -228,7 +241,7 @@ void serv_dspam_shutdown (void)
 
 CTDL_MODULE_INIT(dspam)
 {
-	return "$Id: serv_dspam.c 5876 2007-12-10 23:22:03Z dothebart $" "disabled.";
+	return "$Id: serv_dspam.c 7921 2009-10-17 12:21:17Z davew $" "disabled.";
 	if (!threading)
 	{
 #ifdef HAVE_LIBDSPAM
@@ -249,6 +262,6 @@ CTDL_MODULE_INIT(dspam)
 	}
 	
         /* return our Subversion id for the Log */
-	return "$Id: serv_dspam.c 5876 2007-12-10 23:22:03Z dothebart $";
+	return "$Id: serv_dspam.c 7921 2009-10-17 12:21:17Z davew $";
 }
 
