@@ -1,8 +1,24 @@
 /*
- * $Id: serv_fulltext.c 6447 2008-07-17 12:57:23Z davew $
+ * $Id: serv_fulltext.c 8025 2009-11-10 03:58:32Z ajc $
  *
  * This module handles fulltext indexing of the message base.
  *
+ *
+ * Copyright (c) 2005-2009 by the citadel.org team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 
@@ -39,10 +55,10 @@
 #include "database.h"
 #include "msgbase.h"
 #include "control.h"
-#include "room_ops.h"
 #include "serv_fulltext.h"
 #include "ft_wordbreaker.h"
 #include "threads.h"
+#include "context.h"
 
 #include "ctdl_module.h"
 
@@ -109,6 +125,19 @@ void ft_index_message(long msgnum, int op) {
 	struct cdbdata *cdb_bucket;
 	char *msgtext;
 	int tok;
+	struct CtdlMessage *msg = NULL;
+
+	msg = CtdlFetchMessage(msgnum, 1);
+	if (msg == NULL) {
+		CtdlLogPrintf(CTDL_ERR, "ft_index_message() could not load msg %ld\n", msgnum);
+		return;
+	}
+
+	if (msg->cm_fields['1'] != NULL) {
+		CtdlLogPrintf(CTDL_DEBUG, "ft_index_message() excluded msg %ld\n", msgnum);
+		CtdlFreeMessage(msg);
+		return;
+	}
 
 	CtdlLogPrintf(CTDL_DEBUG, "ft_index_message() %s msg %ld\n",
 		(op ? "adding" : "removing") , msgnum
@@ -120,7 +149,8 @@ void ft_index_message(long msgnum, int op) {
 	CC->redirect_buffer = malloc(SIZ);
 	CC->redirect_len = 0;
 	CC->redirect_alloc = SIZ;
-	CtdlOutputMsg(msgnum, MT_CITADEL, HEADERS_ALL, 0, 1, NULL, 0);
+	CtdlOutputPreLoadedMsg(msg, MT_CITADEL, HEADERS_ALL, 0, 1, 0);
+	CtdlFreeMessage(msg);
 	msgtext = CC->redirect_buffer;
 	CC->redirect_buffer = NULL;
 	CC->redirect_len = 0;
@@ -210,13 +240,13 @@ void ft_index_room(struct ctdlroom *qrbuf, void *data)
 	if (CtdlThreadCheckStop())
 		return;
 		
-	getroom(&CC->room, qrbuf->QRname);
+	CtdlGetRoom(&CC->room, qrbuf->QRname);
 	CtdlForEachMessage(MSGS_ALL, 0L, NULL, NULL, NULL, ft_index_msg, NULL);
 }
 
 
 /*
- * Begin the fulltext indexing process.  (Called as an EVT_TIMER event)
+ * Begin the fulltext indexing process.
  */
 void do_fulltext_indexing(void) {
 	int i;
@@ -274,7 +304,7 @@ void do_fulltext_indexing(void) {
 	 * Now go through each room and find messages to index.
 	 */
 	ft_newhighest = CitControl.MMhighest;
-	ForEachRoom(ft_index_room, NULL);	/* load all msg pointers */
+	CtdlForEachRoom(ft_index_room, NULL);	/* load all msg pointers */
 
 	if (ft_num_msgs > 0) {
 		qsort(ft_newmsgs, ft_num_msgs, sizeof(long), longcmp);
@@ -507,5 +537,5 @@ CTDL_MODULE_INIT(fulltext)
 		CtdlThreadCreate("Indexer", CTDLTHREAD_BIGSTACK, indexer_thread, NULL);
 	}
 	/* return our Subversion id for the Log */
-	return "$Id: serv_fulltext.c 6447 2008-07-17 12:57:23Z davew $";
+	return "$Id: serv_fulltext.c 8025 2009-11-10 03:58:32Z ajc $";
 }
