@@ -1,11 +1,26 @@
 /*
- * $Id: imap_acl.c 7291 2009-04-06 14:54:13Z ajc $
+ * $Id: imap_acl.c 8060 2009-11-18 15:35:41Z ajc $
  *
  * Functions which implement RFC2086 (and maybe RFC4314) (IMAP ACL extension)
  *
+ *
+ * Copyright (c) 2007-2009 by the citadel.org team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-
 #include "sysdep.h"
 #include <stdlib.h>
 #include <unistd.h>
@@ -38,7 +53,6 @@
 #include "citserver.h"
 #include "support.h"
 #include "config.h"
-#include "room_ops.h"
 #include "user_ops.h"
 #include "policy.h"
 #include "database.h"
@@ -49,7 +63,7 @@
 #include "imap_fetch.h"
 #include "imap_misc.h"
 #include "genstamp.h"
-
+#include "ctdl_module.h"
 
 
 /*
@@ -86,7 +100,7 @@ void imap_acl_flags(char *rights, int ra)
 	 */
 	if (	(ra & UA_KNOWN)					/* known rooms */
 	   ||	((ra & UA_GOTOALLOWED) && (ra & UA_ZAPPED))	/* zapped rooms */
-	   ) {
+	) {
 		strcat(rights, "l");
 		strcat(rights, "r");
 		strcat(rights, "s");
@@ -118,8 +132,9 @@ void imap_acl_flags(char *rights, int ra)
 			 * commands are not yet implemented, omitting the flag should
 			 * theoretically prevent compliant clients from attempting to
 			 * perform them.
+			 *
+			 * strcat(rights, "a");
 			 */
-			/* strcat(rights, "a"); * commented out */
 		}
 	}
 }
@@ -148,19 +163,18 @@ void imap_getacl(int num_parms, char *parms[]) {
 	 */
 	ret = imap_grabroom(roomname, parms[2], 1);
 	if (ret != 0) {
-		cprintf("%s NO Invalid mailbox name or access denied\r\n",
-			parms[0]);
+		cprintf("%s NO Invalid mailbox name or access denied\r\n", parms[0]);
 		return;
 	}
 
 	/*
-	 * usergoto() formally takes us to the desired room.  (If another
+	 * CtdlUserGoto() formally takes us to the desired room.  (If another
 	 * folder is selected, save its name so we can return there!!!!!)
 	 */
 	if (IMAP->selected) {
 		strcpy(savedroom, CC->room.QRname);
 	}
-	usergoto(roomname, 0, 0, &msgs, &new);
+	CtdlUserGoto(roomname, 0, 0, &msgs, &new);
 
 	cprintf("* ACL");
 	cprintf(" ");
@@ -193,7 +207,7 @@ void imap_getacl(int num_parms, char *parms[]) {
 	 * our happy day without violent explosions.
 	 */
 	if (IMAP->selected) {
-		usergoto(savedroom, 0, 0, &msgs, &new);
+		CtdlUserGoto(savedroom, 0, 0, &msgs, &new);
 	}
 
 	cprintf("%s OK GETACL completed\r\n", parms[0]);
@@ -221,8 +235,7 @@ void imap_listrights(int num_parms, char *parms[]) {
 	 */
 	ret = imap_grabroom(roomname, parms[2], 1);
 	if (ret != 0) {
-		cprintf("%s NO Invalid mailbox name or access denied\r\n",
-			parms[0]);
+		cprintf("%s NO Invalid mailbox name or access denied\r\n", parms[0]);
 		return;
 	}
 
@@ -233,25 +246,23 @@ void imap_listrights(int num_parms, char *parms[]) {
 	valid = validate_recipients(parms[3], NULL, 0);
 	if (valid != NULL) {
 		if (valid->num_local == 1) {
-			ret = getuser(&temp, valid->recp_local);
+			ret = CtdlGetUser(&temp, valid->recp_local);
 		}
 		free_recipients(valid);
 	}
 	if (ret != 0) {
-		cprintf("%s NO Invalid user name or access denied\r\n",
-			parms[0]);
+		cprintf("%s NO Invalid user name or access denied\r\n", parms[0]);
 		return;
 	}
 
 	/*
-	 * usergoto() formally takes us to the desired room.  (If another
+	 * CtdlUserGoto() formally takes us to the desired room.  (If another
 	 * folder is selected, save its name so we can return there!!!!!)
 	 */
 	if (IMAP->selected) {
 		strcpy(savedroom, CC->room.QRname);
 	}
-	usergoto(roomname, 0, 0, &msgs, &new);
-
+	CtdlUserGoto(roomname, 0, 0, &msgs, &new);
 
 	/*
 	 * Now output the list of rights
@@ -264,13 +275,12 @@ void imap_listrights(int num_parms, char *parms[]) {
 	imap_strout("");		/* FIXME ... do something here */
 	cprintf("\r\n");
 
-
 	/*
 	 * If another folder is selected, go back to that room so we can resume
 	 * our happy day without violent explosions.
 	 */
 	if (IMAP->selected) {
-		usergoto(savedroom, 0, 0, &msgs, &new);
+		CtdlUserGoto(savedroom, 0, 0, &msgs, &new);
 	}
 
 	cprintf("%s OK LISTRIGHTS completed\r\n", parms[0]);
@@ -296,19 +306,18 @@ void imap_myrights(int num_parms, char *parms[]) {
 
 	ret = imap_grabroom(roomname, parms[2], 1);
 	if (ret != 0) {
-		cprintf("%s NO Invalid mailbox name or access denied\r\n",
-			parms[0]);
+		cprintf("%s NO Invalid mailbox name or access denied\r\n", parms[0]);
 		return;
 	}
 
 	/*
-	 * usergoto() formally takes us to the desired room.  (If another
+	 * CtdlUserGoto() formally takes us to the desired room.  (If another
 	 * folder is selected, save its name so we can return there!!!!!)
 	 */
 	if (IMAP->selected) {
 		strcpy(savedroom, CC->room.QRname);
 	}
-	usergoto(roomname, 0, 0, &msgs, &new);
+	CtdlUserGoto(roomname, 0, 0, &msgs, &new);
 
 	CtdlRoomAccess(&CC->room, &CC->user, &ra, NULL);
 	imap_acl_flags(rights, ra);
@@ -321,11 +330,9 @@ void imap_myrights(int num_parms, char *parms[]) {
 	 * If a different folder was previously selected, return there now.
 	 */
 	if ( (IMAP->selected) && (strcasecmp(roomname, savedroom)) ) {
-		usergoto(savedroom, 0, 0, &msgs, &new);
+		CtdlUserGoto(savedroom, 0, 0, &msgs, &new);
 	}
 
 	cprintf("%s OK MYRIGHTS completed\r\n", parms[0]);
 	return;
 }
-
-
