@@ -1,9 +1,23 @@
 /*
- * $Id: serv_pop3.c 6449 2008-07-17 18:36:27Z davew $ 
+ * $Id: serv_pop3.c 7957 2009-10-25 01:07:14Z davew $ 
  *
  * POP3 service for the Citadel system
- * Copyright (C) 1998-2001 by Art Cancro and others.
- * This code is released under the terms of the GNU General Public License.
+ *
+ * Copyright (c) 1998-2009 by the citadel.org team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * Current status of standards conformance:
  *
@@ -47,7 +61,6 @@
 #include "citserver.h"
 #include "support.h"
 #include "config.h"
-#include "room_ops.h"
 #include "user_ops.h"
 #include "policy.h"
 #include "database.h"
@@ -159,7 +172,7 @@ void pop3_add_message(long msgnum, void *userdata) {
 		CC->redirect_buffer = malloc(SIZ);
 		CC->redirect_len = 0;
 		CC->redirect_alloc = SIZ;
-		CtdlOutputMsg(msgnum, MT_RFC822, HEADERS_ALL, 0, 1, NULL, 0);
+		CtdlOutputMsg(msgnum, MT_RFC822, HEADERS_ALL, 0, 1, NULL, SUPPRESS_ENV_TO);
 		smi.meta_rfc822_length = CC->redirect_len;
 		free(CC->redirect_buffer);
 		CC->redirect_buffer = NULL;
@@ -181,7 +194,7 @@ int pop3_grab_mailbox(void) {
         struct visit vbuf;
 	int i;
 
-	if (getroom(&CC->room, MAILROOM) != 0) return(-1);
+	if (CtdlGetRoom(&CC->room, MAILROOM) != 0) return(-1);
 
 	/* Load up the messages */
 	CtdlForEachMessage(MSGS_ALL, 0L, NULL, NULL, NULL,
@@ -252,7 +265,7 @@ void pop3_apop(char *argbuf)
    	return;
    }
    
-   if (getuser(&CC->user, CC->curr_user))
+   if (CtdlGetUser(&CC->user, CC->curr_user))
    {
    	cprintf("-ERR No such user.\r\n");
    	return;
@@ -373,7 +386,10 @@ void pop3_retr(char *argbuf) {
 	}
 
 	cprintf("+OK Message %d:\r\n", which_one);
-	CtdlOutputMsg(POP3->msgs[which_one - 1].msgnum, MT_RFC822, HEADERS_ALL, 0, 1, NULL, ESC_DOT);
+	CtdlOutputMsg(POP3->msgs[which_one - 1].msgnum,
+		MT_RFC822, HEADERS_ALL, 0, 1, NULL,
+		(ESC_DOT|SUPPRESS_ENV_TO)
+	);
 	cprintf(".\r\n");
 }
 
@@ -405,8 +421,7 @@ void pop3_top(char *argbuf) {
 	CC->redirect_buffer = malloc(SIZ);
 	CC->redirect_len = 0;
 	CC->redirect_alloc = SIZ;
-	CtdlOutputMsg(POP3->msgs[which_one - 1].msgnum,
-		      MT_RFC822, HEADERS_ALL, 0, 1, NULL, 0);
+	CtdlOutputMsg(POP3->msgs[which_one - 1].msgnum, MT_RFC822, HEADERS_ALL, 0, 1, NULL, SUPPRESS_ENV_TO);
 	msgtext = CC->redirect_buffer;
 	CC->redirect_buffer = NULL;
 	CC->redirect_len = 0;
@@ -489,14 +504,14 @@ void pop3_update(void) {
 
 	/* Set last read pointer */
 	if (POP3->num_msgs > 0) {
-		lgetuser(&CC->user, CC->curr_user);
+		CtdlGetUserLock(&CC->user, CC->curr_user);
 
 		CtdlGetRelationship(&vbuf, &CC->user, &CC->room);
 		snprintf(vbuf.v_seen, sizeof vbuf.v_seen, "*:%ld",
 			POP3->msgs[POP3->num_msgs-1].msgnum);
 		CtdlSetRelationship(&vbuf, &CC->user, &CC->room);
 
-		lputuser(&CC->user);
+		CtdlPutUserLock(&CC->user);
 	}
 
 }
@@ -741,5 +756,5 @@ CTDL_MODULE_INIT(pop3)
 	}
 	
 	/* return our Subversion id for the Log */
-	return "$Id: serv_pop3.c 6449 2008-07-17 18:36:27Z davew $";
+	return "$Id: serv_pop3.c 7957 2009-10-25 01:07:14Z davew $";
 }
