@@ -1,10 +1,26 @@
 /*
- * $Id: serv_spam.c 6671 2008-10-09 14:08:27Z ajc $
+ * $Id: serv_spam.c 7921 2009-10-17 12:21:17Z davew $
  *
  * This module allows Citadel to use SpamAssassin to filter incoming messages
  * arriving via SMTP.  For more information on SpamAssassin, visit
  * http://www.spamassassin.org (the SpamAssassin project is not in any way
  * affiliated with the Citadel project).
+ *
+ * Copyright (c) 1998-2009 by the citadel.org team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #define SPAMASSASSIN_PORT       "783"
@@ -41,7 +57,6 @@
 #include "support.h"
 #include "config.h"
 #include "control.h"
-#include "room_ops.h"
 #include "user_ops.h"
 #include "policy.h"
 #include "database.h"
@@ -189,5 +204,5 @@ CTDL_MODULE_INIT(spam)
 	}
 	
 	/* return our Subversion id for the Log */
-        return "$Id: serv_spam.c 6671 2008-10-09 14:08:27Z ajc $";
+        return "$Id: serv_spam.c 7921 2009-10-17 12:21:17Z davew $";
 }
