@@ -1,8 +1,23 @@
 /*
- * $Id: serv_upgrade.c 6960 2009-01-18 05:35:55Z ajc $
+ * $Id: serv_upgrade.c 7957 2009-10-25 01:07:14Z davew $
  *
  * Transparently handle the upgrading of server data formats.
  *
+ * Copyright (c) 1987-2009 by the citadel.org team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "sysdep.h"
@@ -37,7 +52,6 @@
 #include "config.h"
 #include "control.h"
 #include "database.h"
-#include "room_ops.h"
 #include "user_ops.h"
 #include "msgbase.h"
 #include "serv_upgrade.h"
@@ -57,12 +71,12 @@ void fix_sys_user_name(void)
 	char usernamekey[USERNAME_SIZE];
 
 	/** If we have a user called Citadel rename them to SYS_Citadel */
-	if (getuser(&usbuf, "Citadel") == 0)
+	if (CtdlGetUser(&usbuf, "Citadel") == 0)
 	{
 		rename_user("Citadel", "SYS_Citadel");
 	}
 
-	while (getuserbynumber(&usbuf, 0) == 0)
+	while (CtdlGetUserByNumber(&usbuf, 0) == 0)
 	{
 		/* delete user with number 0 and no name */
 		if (IsEmptyStr(usbuf.fullname))
@@ -70,12 +84,12 @@ void fix_sys_user_name(void)
 		else
 		{ /* temporarily set this user to -1 */
 			usbuf.usernum = -1;
-			putuser(&usbuf);
+			CtdlPutUser(&usbuf);
 		}
 	}
 
 	/** Make sure user SYS_* is user 0 */
-	while (getuserbynumber(&usbuf, -1) == 0)
+	while (CtdlGetUserByNumber(&usbuf, -1) == 0)
 	{
 		if (strncmp(usbuf.fullname, "SYS_", 4))
 		{	/** Delete any user 0 that doesn't start with SYS_ */
@@ -85,7 +99,7 @@ void fix_sys_user_name(void)
 		else
 		{
 			usbuf.usernum = 0;
-			putuser(&usbuf);
+			CtdlPutUser(&usbuf);
 		}
 	}
 }
@@ -99,7 +113,7 @@ void cmd_bmbx_backend(struct ctdlroom *qrbuf, void *data) {
 	struct RoomProcList *ptr;
 	struct ctdlroom qr;
 
-	/* Lazy programming here.  Call this function as a ForEachRoom backend
+	/* Lazy programming here.  Call this function as a CtdlForEachRoom backend
 	 * in order to queue up the room names, or call it with a null room
 	 * to make it do the processing.
 	 */
@@ -116,7 +130,7 @@ void cmd_bmbx_backend(struct ctdlroom *qrbuf, void *data) {
 
 	while (rplist != NULL) {
 
-		if (lgetroom(&qr, rplist->name) == 0) {
+		if (CtdlGetRoomLock(&qr, rplist->name) == 0) {
 			CtdlLogPrintf(CTDL_DEBUG, "Processing <%s>...\n", rplist->name);
 			if ( (qr.QRflags & QR_MAILBOX) == 0) {
 				CtdlLogPrintf(CTDL_DEBUG, "  -- not a mailbox\n");
@@ -126,7 +140,7 @@ void cmd_bmbx_backend(struct ctdlroom *qrbuf, void *data) {
 				qr.QRgen = time(NULL);
 				CtdlLogPrintf(CTDL_DEBUG, "  -- fixed!\n");
 			}
-			lputroom(&qr);
+			CtdlPutRoomLock(&qr);
 		}
 
 		ptr = rplist;
@@ -140,7 +154,7 @@ void cmd_bmbx_backend(struct ctdlroom *qrbuf, void *data) {
  */
 void bump_mailbox_generation_numbers(void) {
 	CtdlLogPrintf(CTDL_WARNING, "Applying security fix to mailbox rooms\n");
-	ForEachRoom(cmd_bmbx_backend, NULL);
+	CtdlForEachRoom(cmd_bmbx_backend, NULL);
 	cmd_bmbx_backend(NULL, NULL);
 	return;
 }
@@ -171,12 +185,12 @@ void cbtm_backend(struct ctdluser *usbuf, void *data) {
 
 	while (uplist != NULL) {
 
-		if (lgetuser(&us, uplist->user) == 0) {
+		if (CtdlGetUserLock(&us, uplist->user) == 0) {
 			CtdlLogPrintf(CTDL_DEBUG, "Processing <%s>...\n", uplist->user);
 			if (us.uid == CTDLUID) {
 				us.uid = (-1);
 			}
-			lputuser(&us);
+			CtdlPutUserLock(&us);
 		}
 
 		ptr = uplist;
@@ -251,6 +265,17 @@ void update_config(void) {
 		config.c_xmpp_s2s_port = 5269;
 	}
 
+	/* Erase LDAP config that might be left over from the old connector
+	 * which served a different purpose.
+	 */
+	if (CitControl.version < 760) {
+		strcpy(config.c_ldap_host, "");
+		config.c_ldap_port = 389;
+		strcpy(config.c_ldap_base_dn, "");
+		strcpy(config.c_ldap_bind_dn, "");
+		strcpy(config.c_ldap_bind_pw, "");
+	}
+
 	if (IsEmptyStr(config.c_default_cal_zone)) {
 		guess_time_zone();
 	}
@@ -310,5 +335,5 @@ CTDL_MODULE_UPGRADE(upgrade)
 	check_server_upgrades();
 	
 	/* return our Subversion id for the Log */
-	return "$Id: serv_upgrade.c 6960 2009-01-18 05:35:55Z ajc $";
+	return "$Id: serv_upgrade.c 7957 2009-10-25 01:07:14Z davew $";
 }
