@@ -1,10 +1,24 @@
 /*
- * $Id: serv_vcard.c 7214 2009-03-13 02:19:05Z ajc $
+ * $Id: serv_vcard.c 7957 2009-10-25 01:07:14Z davew $
  * 
  * A server-side module for Citadel which supports address book information
  * using the standard vCard format.
  * 
- * Copyright (c) 1999-2007 / released under the GNU General Public License
+ * Copyright (c) 1999-2009 by the citadel.org team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 /*
@@ -54,18 +68,15 @@
 #include "support.h"
 #include "config.h"
 #include "control.h"
-#include "room_ops.h"
 #include "user_ops.h"
 #include "policy.h"
 #include "database.h"
 #include "msgbase.h"
 #include "internet_addressing.h"
 #include "serv_vcard.h"
-
+#include "citadel_ldap.h"
 #include "ctdl_module.h"
 
-
-
 /*
  * set global flag calling for an aide to validate new users
  */
@@ -83,8 +94,7 @@ void set_mm_valid(void) {
  * Extract Internet e-mail addresses from a message containing a vCard, and
  * perform a callback for any found.
  */
-void vcard_extract_internet_addresses(struct CtdlMessage *msg,
-				void (*callback)(char *, char *) ) {
+void vcard_extract_internet_addresses(struct CtdlMessage *msg, void (*callback)(char *, char *) ) {
 	struct vCard *v;
 	char *s;
 	char *k;
@@ -127,199 +137,6 @@ void vcard_extract_internet_addresses(struct CtdlMessage *msg,
 }
 
 
-/* 
- * vCard-to-LDAP conversions.
- *
- * If 'op' is set to V2L_WRITE, then write
- * (add, or change if already exists) a directory entry to the
- * LDAP server, based on the information supplied in a vCard.
- *
- * If 'op' is set to V2L_DELETE, then delete the entry from LDAP.
- */
- 
- 
-void ctdl_vcard_to_directory(struct CtdlMessage *msg, int op) {
-	struct vCard *v = NULL;
-	int i;
-	int have_addr = 0;
-	int have_cn = 0;
-	
-	void *objectlist = NULL;
-
-	char givenname[128];
-	char sn[128];
-	char uid[256];
-	char street[256];
-	char city[128];
-	char state[3];
-	char zipcode[10];
-	char calFBURL[256];
-	char ldap_dn[SIZ];
-
-	if (msg == NULL) return;
-	if (msg->cm_fields['M'] == NULL) return;
-	if (msg->cm_fields['A'] == NULL) return;
-	if (msg->cm_fields['N'] == NULL) return;
-
-	/* Initialize variables */
-	strcpy(givenname, "");
-	strcpy(sn, "");
-	strcpy(calFBURL, "");
-
-	sprintf(uid, "%s@%s",
-		msg->cm_fields['A'],
-		msg->cm_fields['N']
-	);
-
-	sprintf(ldap_dn, "euid=%s,ou=%s", msg->cm_fields['E'], msg->cm_fields['N']);
-	
-	/* Are we just deleting?  If so, it's simple... */
-	if (op == V2L_DELETE) {
-		(void) CtdlDoDirectoryServiceFunc (ldap_dn, NULL, NULL, "ldap", DIRECTORY_USER_DEL);
-		return;
-	}
-
-	/*
-	 * If we get to this point then it must be a V2L_WRITE operation.
-	 */
-
-	/* First make sure the OU for the user's home Citadel host is created */
-	(void) CtdlDoDirectoryServiceFunc (NULL, msg->cm_fields['N'], NULL, "ldap", DIRECTORY_CREATE_HOST);
-	
-	/* Next create the directory service object */
-	(void) CtdlDoDirectoryServiceFunc(NULL, NULL, &objectlist, "ldap", DIRECTORY_CREATE_OBJECT);
-
-	/* The first LDAP attribute will be an 'objectclass' list.  Citadel
-	 * doesn't do anything with this.  It's just there for compatibility
-	 * with Kolab.
-	 */
-	(void) CtdlDoDirectoryServiceFunc("objectclass", "citadelInetOrgPerson", &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-
-	/* Convert the vCard fields to LDAP properties */
-	v = vcard_load(msg->cm_fields['M']);
-	if (v->numprops) for (i=0; i<(v->numprops); ++i) if (striplt(v->prop[i].value), strlen(v->prop[i].value) > 0) {
-
-		if (!strcasecmp(v->prop[i].name, "n")) {
-			extract_token(sn,		v->prop[i].value, 0, ';', sizeof sn);
-			extract_token(givenname,	v->prop[i].value, 1, ';', sizeof givenname);
-		}
-
-		if (!strcasecmp(v->prop[i].name, "fn")) {
-			(void) CtdlDoDirectoryServiceFunc("cn", v->prop[i].value, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-			have_cn = 1;
-		}
-
-		if (!strcasecmp(v->prop[i].name, "title")) {
-			(void) CtdlDoDirectoryServiceFunc("title", v->prop[i].value, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-		}
-
-		if (!strcasecmp(v->prop[i].name, "org")) {
-			(void) CtdlDoDirectoryServiceFunc("o", v->prop[i].value, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-		}
-
-		if ( (!strcasecmp(v->prop[i].name, "adr"))
-		   ||(!strncasecmp(v->prop[i].name, "adr;", 4)) ) {
-			/* Unfortunately, we can only do a single address */
-			if (!have_addr) {
-				have_addr = 1;
-				strcpy(street, "");
-				extract_token(&street[strlen(street)],
-					v->prop[i].value, 0, ';', (sizeof street - strlen(street))); /* po box */
-				strcat(street, " ");
-				extract_token(&street[strlen(street)],
-					v->prop[i].value, 1, ';', (sizeof street - strlen(street))); /* extend addr */
-				strcat(street, " ");
-				extract_token(&street[strlen(street)],
-					v->prop[i].value, 2, ';', (sizeof street - strlen(street))); /* street */
-				striplt(street);
-				extract_token(city, v->prop[i].value, 3, ';', sizeof city);
-				extract_token(state, v->prop[i].value, 4, ';', sizeof state);
-				extract_token(zipcode, v->prop[i].value, 5, ';', sizeof zipcode);
-
-				// ldap requires these fields to be something
-				if (IsEmptyStr(street)) strcpy(street, "_");
-				if (IsEmptyStr(zipcode)) strcpy(zipcode, "_");
-				if (IsEmptyStr(city)) strcpy(city, "_");
-				if (IsEmptyStr(state)) strcpy(state, "_");
-
-				(void) CtdlDoDirectoryServiceFunc("street", street, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-				(void) CtdlDoDirectoryServiceFunc("l", city, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-				(void) CtdlDoDirectoryServiceFunc("st", state, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-				(void) CtdlDoDirectoryServiceFunc("postalcode", zipcode, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-			}
-		}
-
-		if ( (!strcasecmp(v->prop[i].name, "tel;home"))
-		   || (!strcasecmp(v->prop[i].name, "tel;type=home")) )
-			(void) CtdlDoDirectoryServiceFunc("homePhone", v->prop[i].value, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-		else
-		if ( (!strcasecmp(v->prop[i].name, "tel;fax"))
-		   || (!strcasecmp(v->prop[i].name, "tel;type=fax")) )
-			(void) CtdlDoDirectoryServiceFunc("facsimileTelephoneNumber", v->prop[i].value, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-		else
-		if ( (!strcasecmp(v->prop[i].name, "tel;cell"))
-		   || (!strcasecmp(v->prop[i].name, "tel;type=cell")) )
-			(void) CtdlDoDirectoryServiceFunc("mobile", v->prop[i].value, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-		else
-		if ( (!strcasecmp(v->prop[i].name, "tel"))
-		   ||(!strncasecmp(v->prop[i].name, "tel;", 4)) ) {
-			(void) CtdlDoDirectoryServiceFunc("telephoneNumber", v->prop[i].value, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-		}
-
-
-		if ( (!strcasecmp(v->prop[i].name, "email"))
-		   ||(!strcasecmp(v->prop[i].name, "email;internet")) ) {
-			(void) CtdlDoDirectoryServiceFunc("mail", v->prop[i].value, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-		}
-
-		/* Calendar free/busy URL (take the first one we find, but if a subsequent
-		 * one contains the "pref" designation then we go with that instead.)
-		 */
-		if ( (!strcasecmp(v->prop[i].name, "fburl"))
-		   ||(!strncasecmp(v->prop[i].name, "fburl;", 6)) ) {
-			if ( (IsEmptyStr(calFBURL))
-			   || (!strncasecmp(v->prop[i].name, "fburl;pref", 10)) ) {
-				safestrncpy(calFBURL, v->prop[i].value, sizeof calFBURL);
-			}
-		}
-
-	}
-	vcard_free(v);	/* Don't need this anymore. */
-
-	/* "sn" (surname) based on info in vCard */
-	(void) CtdlDoDirectoryServiceFunc("sn", sn, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-
-	/* "givenname" (first name) based on info in vCard */
-	if (IsEmptyStr(givenname)) strcpy(givenname, "_");
-	if (IsEmptyStr(sn)) strcpy(sn, "_");
-	(void) CtdlDoDirectoryServiceFunc("givenname", givenname, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-
-	/* "uid" is a Kolab compatibility thing.  We just do cituser@citnode */
-	(void) CtdlDoDirectoryServiceFunc("uid", uid, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-
-	/* Add a "cn" (Common Name) attribute based on the user's screen name,
-	 * but only there was no 'fn' (full name) property in the vCard	
-	 */
-	if (!have_cn)
-		(void) CtdlDoDirectoryServiceFunc("cn", msg->cm_fields['A'], &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-
-	/* Add a "calFBURL" attribute if a calendar free/busy URL exists */
-	if (!IsEmptyStr(calFBURL)) {
-		(void) CtdlDoDirectoryServiceFunc("calFBURL", calFBURL, &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-	}
-	
-	// Add this messages EUID as the primary key for this entry.
-	(void) CtdlDoDirectoryServiceFunc("euid", msg->cm_fields['E'], &objectlist, "ldap", DIRECTORY_ATTRIB_ADD);
-	
-	
-	(void) CtdlDoDirectoryServiceFunc(ldap_dn, NULL, &objectlist, "ldap", DIRECTORY_SAVE_OBJECT);
-	
-	(void) CtdlDoDirectoryServiceFunc(NULL, NULL, &objectlist, "ldap", DIRECTORY_FREE_OBJECT);
-	CtdlLogPrintf(CTDL_DEBUG, "Directory Services write operation complete.\n");
-}
-
-
-
 /*
  * Callback for vcard_add_to_directory()
  * (Lotsa ugly nested callbacks.  Oh well.)
@@ -343,8 +160,7 @@ void vcard_directory_add_user(char *internet_addr, char *citadel_addr) {
 			}
 		}
 	}
-	CtdlLogPrintf(CTDL_INFO, "Adding %s (%s) to directory\n",
-			citadel_addr, internet_addr);
+	CtdlLogPrintf(CTDL_INFO, "Adding %s (%s) to directory\n", citadel_addr, internet_addr);
 	CtdlDirectoryAddUser(internet_addr, citadel_addr);
 }
 
@@ -360,8 +176,6 @@ void vcard_add_to_directory(long msgnum, void *data) {
 		vcard_extract_internet_addresses(msg, vcard_directory_add_user);
 	}
 
-	ctdl_vcard_to_directory(msg, V2L_WRITE);
-
 	CtdlFreeMessage(msg);
 }
 
@@ -376,8 +190,8 @@ void cmd_igab(char *argbuf) {
 
 	strcpy(hold_rm, CC->room.QRname);	/* save current room */
 
-	if (getroom(&CC->room, ADDRESS_BOOK_ROOM) != 0) {
-		getroom(&CC->room, hold_rm);
+	if (CtdlGetRoom(&CC->room, ADDRESS_BOOK_ROOM) != 0) {
+		CtdlGetRoom(&CC->room, hold_rm);
 		cprintf("%d cannot get address book room\n", ERROR + ROOM_NOT_FOUND);
 		return;
 	}
@@ -390,7 +204,7 @@ void cmd_igab(char *argbuf) {
 	CtdlForEachMessage(MSGS_ALL, 0, NULL, "^[Tt][Ee][Xx][Tt]/.*[Vv][Cc][Aa][Rr][Dd]$",
 		NULL, vcard_add_to_directory, NULL);
 
-	getroom(&CC->room, hold_rm);	/* return to saved room */
+	CtdlGetRoom(&CC->room, hold_rm);	/* return to saved room */
 	cprintf("%d Directory has been rebuilt.\n", CIT_OK);
 }
 
@@ -565,7 +379,7 @@ int vcard_upload_beforesave(struct CtdlMessage *msg) {
 			memcpy(&usbuf, &CC->user, sizeof(struct ctdluser));
 		}
 		
-		else if (getuserbynumber(&usbuf, what_user) == 0) {
+		else if (CtdlGetUserByNumber(&usbuf, what_user) == 0) {
 			/* We fetched a valid user record */
 		}
 
@@ -686,7 +500,7 @@ int vcard_upload_aftersave(struct CtdlMessage *msg) {
 	if ( (strlen(CC->room.QRname) >= 12) && (!strcasecmp(&CC->room.QRname[11], USERCONFIGROOM)) ) {
 		is_UserConf = 1;	/* It's someone's config room */
 	}
-	MailboxName(roomname, sizeof roomname, &CC->user, USERCONFIGROOM);
+	CtdlMailboxName(roomname, sizeof roomname, &CC->user, USERCONFIGROOM);
 	if (!strcasecmp(CC->room.QRname, roomname)) {
 		is_UserConf = 1;
 		is_MY_UserConf = 1;	/* It's MY config room */
@@ -739,18 +553,18 @@ int vcard_upload_aftersave(struct CtdlMessage *msg) {
 			 * Assume they don't need validating.
 			 */
 			if (CC->user.axlevel >= 6) {
-				lgetuser(&CC->user, CC->curr_user);
+				CtdlGetUserLock(&CC->user, CC->curr_user);
 				CC->user.flags |= US_REGIS;
-				lputuser(&CC->user);
+				CtdlPutUserLock(&CC->user);
 				return (0);
 			}
 			
 			set_mm_valid();
 
 			/* ...which also means we need to flag the user */
-			lgetuser(&CC->user, CC->curr_user);
+			CtdlGetUserLock(&CC->user, CC->curr_user);
 			CC->user.flags |= (US_REGIS|US_NEEDVALID);
-			lputuser(&CC->user);
+			CtdlPutUserLock(&CC->user);
 
 			return(0);
 		}
@@ -787,10 +601,10 @@ struct vCard *vcard_get_user(struct ctdluser *u) {
 	long VCmsgnum;
 
 	strcpy(hold_rm, CC->room.QRname);	/* save current room */
-	MailboxName(config_rm, sizeof config_rm, u, USERCONFIGROOM);
+	CtdlMailboxName(config_rm, sizeof config_rm, u, USERCONFIGROOM);
 
-	if (getroom(&CC->room, config_rm) != 0) {
-		getroom(&CC->room, hold_rm);
+	if (CtdlGetRoom(&CC->room, config_rm) != 0) {
+		CtdlGetRoom(&CC->room, hold_rm);
 		return vcard_new();
 	}
 
@@ -798,7 +612,7 @@ struct vCard *vcard_get_user(struct ctdluser *u) {
 	VCmsgnum = (-1);
 	CtdlForEachMessage(MSGS_LAST, 1, NULL, "^[Tt][Ee][Xx][Tt]/.*[Vv][Cc][Aa][Rr][Dd]$",
 		NULL, vcard_gu_backend, (void *)&VCmsgnum );
-	getroom(&CC->room, hold_rm);	/* return to saved room */
+	CtdlGetRoom(&CC->room, hold_rm);	/* return to saved room */
 
 	if (VCmsgnum < 0L) return vcard_new();
 
@@ -940,7 +754,7 @@ void cmd_greg(char *argbuf)
 		return;
 	}
 
-	if (getuser(&usbuf, who) != 0) {
+	if (CtdlGetUser(&usbuf, who) != 0) {
 		cprintf("%d '%s' not found.\n", ERROR + NO_SUCH_USER, who);
 		return;
 	}
@@ -1140,7 +954,6 @@ void vcard_delete_remove(char *room, long msgnum) {
 		   || (!strncasecmp(ptr, "Content-type: text/vcard", 24)) ) {
 			/* Bingo!  A vCard is being deleted. */
 			vcard_extract_internet_addresses(msg, CtdlDirectoryDelUser);
-			ctdl_vcard_to_directory(msg, V2L_DELETE);
 		}
 		ptr = strchr((char *)ptr, '\n');
 		if (ptr != NULL) ++ptr;
@@ -1348,22 +1161,22 @@ void check_get_greeting(void) {
 /*
  * We don't know if the Contacts room exists so we just create it at login
  */
-void vcard_create_room(void)
+void vcard_CtdlCreateRoom(void)
 {
 	struct ctdlroom qr;
 	struct visit vbuf;
 
 	/* Create the calendar room if it doesn't already exist */
-	create_room(USERCONTACTSROOM, 4, "", 0, 1, 0, VIEW_ADDRESSBOOK);
+	CtdlCreateRoom(USERCONTACTSROOM, 4, "", 0, 1, 0, VIEW_ADDRESSBOOK);
 
 	/* Set expiration policy to manual; otherwise objects will be lost! */
-	if (lgetroom(&qr, USERCONTACTSROOM)) {
+	if (CtdlGetRoomLock(&qr, USERCONTACTSROOM)) {
 		CtdlLogPrintf(CTDL_ERR, "Couldn't get the user CONTACTS room!\n");
 		return;
 	}
 	qr.QRep.expire_mode = EXPIRE_MANUAL;
 	qr.QRdefaultview = VIEW_ADDRESSBOOK;	/* 2 = address book view */
-	lputroom(&qr);
+	CtdlPutRoomLock(&qr);
 
 	/* Set the view to a calendar view */
 	CtdlGetRelationship(&vbuf, &CC->user, &qr);
@@ -1381,15 +1194,41 @@ void vcard_create_room(void)
  */
 void vcard_session_login_hook(void) {
 	struct vCard *v = NULL;
+	struct CitContext *CCC = CC;		/* put this on the stack, just for speed */
 
-	v = vcard_get_user(&CC->user);
-	extract_inet_email_addrs(CC->cs_inet_email, sizeof CC->cs_inet_email,
-				CC->cs_inet_other_emails, sizeof CC->cs_inet_other_emails,
-				v, 1);
-	extract_friendly_name(CC->cs_inet_fn, sizeof CC->cs_inet_fn, v);
-	vcard_free(v);
+#ifdef HAVE_LDAP
+	/*
+	 * Is this an LDAP session?  If so, copy various LDAP attributes from the directory entry
+	 * into the user's vCard.
+	 */
+	if ((config.c_auth_mode == AUTHMODE_LDAP) || (config.c_auth_mode == AUTHMODE_LDAP_AD)) {
+		v = vcard_get_user(&CCC->user);
+		if (v) {
+			if (Ctdl_LDAP_to_vCard(CCC->ldap_dn, v)) {
+				vcard_write_user(&CCC->user, v);
+			}
+		}
+	}
+#endif
 
-	vcard_create_room();
+	/*
+	 * Extract from the user's vCard, any Internet email addresses and the user's real name.
+	 * These are inserted into the session data for various message entry commands to use.
+	 */
+	v = vcard_get_user(&CCC->user);
+	if (v) {
+		extract_inet_email_addrs(CCC->cs_inet_email, sizeof CCC->cs_inet_email,
+					CCC->cs_inet_other_emails, sizeof CCC->cs_inet_other_emails,
+					v, 1
+		);
+		extract_friendly_name(CCC->cs_inet_fn, sizeof CCC->cs_inet_fn, v);
+		vcard_free(v);
+	}
+
+	/*
+	 * Create the user's 'Contacts' room (personal address book) if it doesn't already exist.
+	 */
+	vcard_CtdlCreateRoom();
 }
 
 
@@ -1478,7 +1317,7 @@ void store_this_ha(struct addresses_to_be_filed *aptr) {
 	int i;
 
 	/* First remove any addresses we already have in the address book */
-	usergoto(aptr->roomname, 0, 0, NULL, NULL);
+	CtdlUserGoto(aptr->roomname, 0, 0, NULL, NULL);
 	CtdlForEachMessage(MSGS_ALL, 0, NULL, "^[Tt][Ee][Xx][Tt]/.*[Vv][Cc][Aa][Rr][Dd]$", NULL,
 		strip_addresses_already_have, aptr->collected_addresses);
 
@@ -1574,6 +1413,7 @@ CTDL_MODULE_INIT(vcard)
 	struct ctdlroom qr;
 	char filename[256];
 	FILE *fp;
+	int rv = 0;
 
 	if (!threading)
 	{
@@ -1583,8 +1423,7 @@ CTDL_MODULE_INIT(vcard)
 		CtdlRegisterDeleteHook(vcard_delete_remove);
 		CtdlRegisterProtoHook(cmd_regi, "REGI", "Enter registration info");
 		CtdlRegisterProtoHook(cmd_greg, "GREG", "Get registration info");
-		CtdlRegisterProtoHook(cmd_igab, "IGAB",
-						"Initialize Global Address Book");
+		CtdlRegisterProtoHook(cmd_igab, "IGAB", "Initialize Global Address Book");
 		CtdlRegisterProtoHook(cmd_qdir, "QDIR", "Query Directory");
 		CtdlRegisterProtoHook(cmd_gvsn, "GVSN", "Get Valid Screen Names");
 		CtdlRegisterProtoHook(cmd_gvea, "GVEA", "Get Valid Email Addresses");
@@ -1597,13 +1436,13 @@ CTDL_MODULE_INIT(vcard)
 		CtdlRegisterFixedOutputHook("text/vcard", vcard_fixed_output);
 
 		/* Create the Global ADdress Book room if necessary */
-		create_room(ADDRESS_BOOK_ROOM, 3, "", 0, 1, 0, VIEW_ADDRESSBOOK);
+		CtdlCreateRoom(ADDRESS_BOOK_ROOM, 3, "", 0, 1, 0, VIEW_ADDRESSBOOK);
 
 		/* Set expiration policy to manual; otherwise objects will be lost! */
-		if (!lgetroom(&qr, ADDRESS_BOOK_ROOM)) {
+		if (!CtdlGetRoomLock(&qr, ADDRESS_BOOK_ROOM)) {
 			qr.QRep.expire_mode = EXPIRE_MANUAL;
 			qr.QRdefaultview = VIEW_ADDRESSBOOK;	/* 2 = address book view */
-			lputroom(&qr);
+			CtdlPutRoomLock(&qr);
 
 			/*
 			 * Also make sure it has a netconfig file, so the networker runs
@@ -1614,7 +1453,7 @@ CTDL_MODULE_INIT(vcard)
 			assoc_file_name(filename, sizeof filename, &qr, ctdl_netcfg_dir);
 			fp = fopen(filename, "a");
 			if (fp != NULL) fclose(fp);
-			chown(filename, CTDLUID, (-1));
+			rv = chown(filename, CTDLUID, (-1));
 		}
 
 		/* for postfix tcpdict */
@@ -1627,5 +1466,5 @@ CTDL_MODULE_INIT(vcard)
 	}
 	
 	/* return our Subversion id for the Log */
-	return "$Id: serv_vcard.c 7214 2009-03-13 02:19:05Z ajc $";
+	return "$Id: serv_vcard.c 7957 2009-10-25 01:07:14Z davew $";
 }
