@@ -1,5 +1,5 @@
 /*
- * $Id: serv_extensions.c 7144 2009-02-21 14:41:50Z dothebart $
+ * $Id: serv_extensions.c 7827 2009-09-28 16:40:47Z ajc $
  *
  * Citadel Dynamic Loading Module
  * Written by Brian Costello <btx@calyx.net>
@@ -46,25 +46,13 @@ struct SearchFunctionHook *SearchFunctionHookTable = NULL;
 
 struct ProtoFunctionHook {
 	void (*handler) (char *cmdbuf);
-	char *cmd;
-	char *desc;
-	struct ProtoFunctionHook *next;
-} *ProtoHookList = NULL;
-
-
-struct DirectoryServiceHook {
-	int (*handler) (char *cn, char *ou, void **object);
-	int cmd;
-	char *module;
-	struct DirectoryServiceHook *next;
-} *DirectoryServiceHookList = NULL;
-
-struct DirectoryObject {
-	char *module;
-	void *object;
-	struct DirectoryObject *next;
+	const char *cmd;
+	const char *desc;
 };
 
+HashList *ProtoHookList = NULL;
+
+
 #define ERR_PORT (1 << 1)
 
 
@@ -193,22 +181,41 @@ void AddPortError(char *Port, char *ErrorMessage)
 
 int DLoader_Exec_Cmd(char *cmdbuf)
 {
+	void *vP;
 	struct ProtoFunctionHook *p;
 
-	for (p = ProtoHookList; p; p = p->next) {
-		if (!strncasecmp(cmdbuf, p->cmd, 4)) {
-			p->handler(&cmdbuf[5]);
-			return 1;
-		}
+	if (GetHash(ProtoHookList, cmdbuf, 4, &vP) && (vP != NULL)) {
+		p = (struct ProtoFunctionHook*) vP;
+		p->handler(&cmdbuf[5]);
+		return 1;
 	}
 	return 0;
 }
 
+int FourHash(const char *key, long length) 
+{
+	int i;
+	int ret = 0;
+	const unsigned char *ptr = (const unsigned char*)key;
+
+	for (i = 0; i < 4; i++, ptr ++) 
+		ret = (ret << 8) | 
+			( ((*ptr >= 'a') &&
+			   (*ptr <= 'z'))? 
+			  *ptr - 'a' + 'A': 
+			  *ptr);
+
+	return ret;
+}
 
 void CtdlRegisterProtoHook(void (*handler) (char *), char *cmd, char *desc)
 {
 	struct ProtoFunctionHook *p;
 
+	if (ProtoHookList == NULL)
+		ProtoHookList = NewHash (1, FourHash);
+
+
 	p = (struct ProtoFunctionHook *)
 		malloc(sizeof(struct ProtoFunctionHook));
 
@@ -219,56 +226,15 @@ void CtdlRegisterProtoHook(void (*handler) (char *), char *cmd, char *desc)
 	p->handler = handler;
 	p->cmd = cmd;
 	p->desc = desc;
-	p->next = ProtoHookList;
-	ProtoHookList = p;
+
+	Put(ProtoHookList, cmd, 4, p, NULL);
 	CtdlLogPrintf(CTDL_INFO, "Registered server command %s (%s)\n", cmd, desc);
 }
 
-
-void CtdlUnregisterProtoHook(void (*handler) (char *), char *cmd)
-{
-	struct ProtoFunctionHook *cur = NULL;
-	struct ProtoFunctionHook *p = NULL;
-	struct ProtoFunctionHook *lastcur = NULL;
-
-	for (cur = ProtoHookList; 
-	     cur != NULL; 
-	     cur = (cur != NULL)? cur->next: NULL) {
-		/* This will also remove duplicates if any */
-		while (cur != NULL &&
-				handler == cur->handler &&
-				!strcmp(cmd, cur->cmd)) {
-			CtdlLogPrintf(CTDL_INFO, "Unregistered server command %s (%s)\n",
-					cmd, cur->desc);
-			p = cur->next;
-			if (cur == ProtoHookList) {
-				ProtoHookList = p;
-			}
-			else if (lastcur != NULL)
-			{
-				lastcur->next = p;
-			}
-			free(cur);
-			cur = p;
-		}
-		lastcur = cur;
-	}
-}
-
 void CtdlDestroyProtoHooks(void)
 {
-	struct ProtoFunctionHook *cur, *p;
 
-	cur = ProtoHookList; 
-	while (cur != NULL)
-	{
-		CtdlLogPrintf(CTDL_INFO, "Destroyed server command %s (%s)\n",
-			cur->cmd, cur->desc);
-		p = cur->next;
-		free(cur);
-		cur = p;
-	}
-	ProtoHookList = NULL;
+	DeleteHash(&ProtoHookList);
 }
 
 
@@ -1045,134 +1011,6 @@ int PerformXmsgHooks(char *sender, char *sender_email, char *recp, char *msg)
 }
 
 
-
-
-int CtdlRegisterDirectoryServiceFunc(int (*func)(char *cn, char *ou, void **object), int cmd, char *module)
-{
-	struct DirectoryServiceHook *newfcn;
-	
-	newfcn = DirectoryServiceHookList;
-	while (newfcn)
-	{
-		if (newfcn->cmd == cmd && !strcmp(newfcn->module, module))
-		{
-			CtdlLogPrintf(CTDL_ERR, "Directory service function already handled by module %s\n", module);
-			return -1;
-		}
-		newfcn = newfcn->next;
-	}
-	
-	newfcn = (struct DirectoryServiceHook *) malloc (sizeof(struct DirectoryServiceHook));
-	newfcn->handler = func;
-	newfcn->cmd = cmd;
-	newfcn->module = module;
-	newfcn->next = DirectoryServiceHookList;
-	DirectoryServiceHookList = newfcn;
-	
-	CtdlLogPrintf(CTDL_INFO, "Registered a new directory service function from module %s\n", module);
-	return 0;
-}
-
-int CtdlDoDirectoryServiceFunc(char *cn, char *ou, void **object, char *module, int cmd)
-{
-	struct DirectoryServiceHook *curfcn;
-	struct DirectoryObject *our_object_list = NULL;
-	struct DirectoryObject *newobject = NULL;
-	struct DirectoryObject *oldobject = NULL;
-	
-	
-	curfcn = DirectoryServiceHookList;
-	if (object)
-		our_object_list = (struct DirectoryObject *) *object;
-	
-	while (curfcn)
-	{
-		if (curfcn->cmd == cmd)
-		{
-			if (!module)
-			{
-				if (cmd == DIRECTORY_CREATE_OBJECT)
-				{
-					newobject = (struct DirectoryObject*) malloc (sizeof(struct DirectoryObject));
-					newobject->module = curfcn->module;
-					newobject->object = NULL;
-					newobject->next = our_object_list;
-					our_object_list = newobject;
-				}
-				if (our_object_list)
-				{
-					for(newobject = our_object_list; newobject; newobject=newobject->next)
-					{
-						if (!strcmp(newobject->module, curfcn->module))
-							(void) curfcn->handler(cn, ou, &newobject->object);
-					}
-				}
-				else
-					(void) curfcn->handler(cn, ou, NULL);
-
-				continue;
-			}
-			else 
-			{
-				if(!strcmp(curfcn->module, module))
-				{
-					if (cmd == DIRECTORY_CREATE_OBJECT)
-					{
-						newobject = (struct DirectoryObject*) malloc (sizeof(struct DirectoryObject));
-						newobject->module = module;
-						newobject->object = NULL;
-						newobject->next = our_object_list;
-						our_object_list = newobject;
-					}
-					if (our_object_list)
-					{
-						for(newobject = our_object_list; newobject; newobject=newobject->next)
-						{
-							if (!strcmp(newobject->module, curfcn->module))
-								(void) curfcn->handler(cn, ou, &newobject->object);
-						}
-					}
-					else
-						(void) (curfcn->handler(cn, ou, NULL));
-
-					break;
-				}
-			}
-		}
-		curfcn=curfcn->next;
-	}
-	if (our_object_list)
-	{
-		*object = our_object_list;
-		if (cmd == DIRECTORY_FREE_OBJECT)
-		{	// The objects pointed to by the list should have been freed by the module that created it
-			for(newobject = our_object_list; newobject; )
-			{
-				oldobject=newobject;
-				newobject=newobject->next;
-				free(oldobject);
-			}
-			*object=NULL;
-		}
-	}
-	return 0;
-}
-
-void CtdlDestroyDirectoryServiceFuncs(void)
-{
-	struct DirectoryServiceHook *cur, *next;
-	
-	cur = DirectoryServiceHookList;
-	DirectoryServiceHookList = NULL;
-	while (cur)
-	{
-		next = cur->next;
-		CtdlLogPrintf(CTDL_INFO, "Destroyed directory service function for module %s.\n", cur->module);
-		free(cur);
-		cur = next;
-	}
-}
-
 /*
  * Dirty hack until we impliment a hook mechanism for this
  */
