@@ -1,4 +1,4 @@
-/* $Id:$ */
+/* $Id: threads.h 7994 2009-11-02 13:15:52Z davew $ */
 
 #ifndef THREADS_H
 #define THREADS_H
@@ -126,6 +126,7 @@ struct ThreadTSD {
 
 extern double CtdlThreadLoadAvg;
 extern double CtdlThreadWorkerAvg;
+extern long statcount;		/* are we doing a stats check? */
 extern citthread_key_t ThreadKey;
 
 void ctdl_thread_internal_init_tsd(void);
@@ -136,6 +137,7 @@ void ctdl_thread_internal_calc_loadavg(void);
 void ctdl_thread_internal_free_tsd(void);
 CtdlThreadNode *ctdl_internal_create_thread(char *name, long flags, void *(*thread_func) (void *arg), void *args);
 void ctdl_thread_internal_check_scheduled(void);
+void ctdl_thread_internal_change_state (CtdlThreadNode *this_thread, enum CtdlThreadState new_state);
 
 void InitialiseSemaphores(void);
 int try_critical_section (int which_one);
